#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Mar  2 10:14:03 2021

Sidechain-Core Enumeration:

#1. D_Pi_A_Enumeration(cores1_smis, chains_smis, cores2_smis, verbse=False)
Return the combinational molecular library as a list of rdkit molecules,
cores1_chains_cores2

Examples of args:
    cores1_smis = ['*[n+]1cc[nH]c1', '[n+]1c(*)c[nH]c1', '[n+]1cc[nH]c1(*)']
    chains_smis = ['C[Po]', 'CC[Po]', 'CCC[Po]'] 
    cores2_smis = ['[n+]1ccccc1C(=O)O', 'c1cnccc1C(=O)O', 'c1ccncc1C(=O)O']

Note: 1. Frag core1 must have only 1 position label by [#0];
      2. Frag chain must replace the [#0] atom in core1 from left ending, 
         and the right ending is labeled by [Po] atom;
      3. Frag core2 must replace the [Po] atom in core1_chain to form core1_chain_core2.

By restrict that the frags have 1 and only 1 attaching points, 
we make things simple and controlable.

#2. Cores_Chains_Enumeration(cores_smis, chains_smis, verbse=False)
Return:
    cores_chains

Examples of args:
    cores_smis = ['*[n+]1cc[nH]c1', '[n+]1c(*)c[nH]c1', '[n+]1cc[nH]c1(*)']
    chains_smis = ['C[Po]', 'CC[Po]', 'CCC[Po]'] 


#3. Cores_Chains_Enumeration_FullSubstitute(cores_smis, chains_smis, verbse=False)
Return: full subsitituted version of molecules. 
    cores_chains

Examples of args:
    cores_smis = ['*[n+]1c(*)c[nH]c1(*)', 'n1cc(*)[nH]c1(*)']
    chains_smis = ['CN', 'CCN', 'CCCN'] 
    
#4. D_A_Enumeration_new(ligandA_smis, ligandB_smis, verbse=False)

#5. D_Pi_A_Enumeration_new(ligandA_smis, pi_bridge_smis, ligandB_smis, verbse=False)

#6. D_A_D_Enumeration_new(donor_smis, acceptorAsPiBridge_smis, verbse=False)

#7. D_Pi_A_Enumeration_new_exchange_pibridge_dummy_po(ligandA_smis, pi_bridge_smis, ligandB_smis, verbse=False)

#8. D_A_D_Enumeration_new_exchange_acceptoraspibridge_dummy_po(donor_smis, acceptorAsPiBridge_smis, verbse=False)


@author: tucy
"""

from rdkit import Chem
from rdkit.Chem import Draw
from rdkit.Chem import AllChem
from rdkit import RDLogger
import numpy as np

import subprocess
import time
from joblib import Parallel, delayed
from tqdm import tqdm

RDLogger.DisableLog('rdApp.*') 


#-----------------------------------------------------------------------------
def delete_repeated_smiles_of_mols(mols,is_sanitize=True):
    '''
    delete the repeated smiles within a list of molecules.
    
    Parameters
    ----------
    mols : list of rdkit mols
        DESCRIPTION.

    Returns
    -------
    mols_rev : list of rdkit mols
        DESCRIPTION.

    '''
    mols_smis = [Chem.MolToSmiles(mol) for mol in  mols]
    mols_smis_list = list( set(mols_smis) )
    mols_rev = [Chem.MolFromSmiles(smi,sanitize=is_sanitize) for smi in  mols_smis_list]    
    return mols_rev


def delete_repeated_smiles_of_smis(smis,is_sanitize=True):
    '''
    delete the repeated smiles within a list of smiles of molecules.
    
    Parameters
    ----------
    smis : list of smiles of molecules
        DESCRIPTION.

    Returns
    -------
    smis_rev : list of smiles of molecules
        DESCRIPTION.

    '''
    new_smis = [Chem.MolToSmiles( Chem.MolFromSmiles(smi, sanitize=is_sanitize) ) for smi in  smis]
    smis_rev = list( set(new_smis) )
    
    return smis_rev


def find_label_atom_idx(ligand, atom_symbol='*'):
    atom_idx = 0
    for i in range( ligand.GetNumAtoms() ) :
        #print( i, ligand.GetAtomWithIdx(i).GetSymbol() ) 
        if ligand.GetAtomWithIdx(i).GetSymbol() == atom_symbol:
            atom_idx = i
    return atom_idx


def find_label_atom_idxs(ligand, atom_symbol='*'):
    atom_idxs = []
    for i in range( ligand.GetNumAtoms() ) :
        #print( i, ligand.GetAtomWithIdx(i).GetSymbol() ) 
        if ligand.GetAtomWithIdx(i).GetSymbol() == atom_symbol:
            #atom_idx = i
            atom_idxs.append(i)
    return atom_idxs



# def delete_label_atom(mol, delete_atom_symbol='Po'):
#     rwmol = Chem.RWMol(mol)
#     rwmol.UpdatePropertyCache(strict=False)
    
#     Po_idx = find_label_atom_idx(rwmol, atom_symbol=delete_atom_symbol)
#     #print("Po_idx = ", Po_idx)
    
#     Po_atom = rwmol.GetAtomWithIdx(Po_idx)

#     nbrs = Po_atom.GetNeighbors()
#     #print("Po_atom.GetNeighbors() are: ", Po_atom.GetNeighbors() )
#     if len(nbrs) >= 2:
#         left_atom_idx = nbrs[0].GetIdx()
#         right_atom_idx = nbrs[1].GetIdx()
#         rwmol.RemoveBond(left_atom_idx, Po_idx)
#         rwmol.RemoveBond(right_atom_idx, Po_idx)
#         rwmol.AddBond(left_atom_idx, right_atom_idx, Chem.BondType.SINGLE)
#         rwmol.RemoveAtom(Po_idx)
        
#     return Chem.Mol(rwmol)


def delete_label_atom(mol, delete_atom_symbol='Po', bondtype=Chem.BondType.SINGLE):
    rwmol = Chem.RWMol(mol)
    rwmol.UpdatePropertyCache(strict=False)
    
    Po_idx = find_label_atom_idx(rwmol, atom_symbol=delete_atom_symbol)
    #print("Po_idx = ", Po_idx)
    
    Po_atom = rwmol.GetAtomWithIdx(Po_idx)

    nbrs = Po_atom.GetNeighbors()
    #print("Po_atom.GetNeighbors() are: ", Po_atom.GetNeighbors() )
    if len(nbrs) >= 2:
        left_atom_idx = nbrs[0].GetIdx()
        right_atom_idx = nbrs[1].GetIdx()
        rwmol.RemoveBond(left_atom_idx, Po_idx)
        rwmol.RemoveBond(right_atom_idx, Po_idx)
        rwmol.AddBond(left_atom_idx, right_atom_idx, bondtype)
        rwmol.RemoveAtom(Po_idx)
    elif len(nbrs) == 1:
        left_atom_idx = nbrs[0].GetIdx()
        rwmol.RemoveBond(left_atom_idx, Po_idx)
        rwmol.RemoveAtom(Po_idx)
    else:
        print("###ERROR! The labelled atom has no neighbors exist, please check.")
        
    return Chem.Mol(rwmol)


def delete_label_atoms(mol, delete_atom_symbol='Po'):
    rwmol = Chem.RWMol(mol)
    rwmol.UpdatePropertyCache(strict=False)
    
    Po_idxs = find_label_atom_idxs(rwmol, atom_symbol=delete_atom_symbol)
    #print("Po_idx = ", Po_idx)
    
    for Po_idx in Po_idxs:
        Po_atom = rwmol.GetAtomWithIdx(Po_idx)
    
        nbrs = Po_atom.GetNeighbors()
        #print("Po_atom.GetNeighbors() are: ", Po_atom.GetNeighbors() )
        if len(nbrs) >= 2:
            left_atom_idx = nbrs[0].GetIdx()
            right_atom_idx = nbrs[1].GetIdx()
            rwmol.RemoveBond(left_atom_idx, Po_idx)
            rwmol.RemoveBond(right_atom_idx, Po_idx)
            rwmol.AddBond(left_atom_idx, right_atom_idx, Chem.BondType.SINGLE)
            rwmol.RemoveAtom(Po_idx)
        
    return Chem.Mol(rwmol)


def delete_label_atoms_new(mol, delete_atom_symbol='Po'):
    rwmol = Chem.RWMol(mol)
    rwmol.UpdatePropertyCache(strict=False)
    
    Po_idxs = find_label_atom_idxs(rwmol, atom_symbol=delete_atom_symbol)
    
    if (len(Po_idxs) >= 1):
        for i in range( len(Po_idxs) ):
            rwmol = delete_label_atom(rwmol, delete_atom_symbol='Po')
        
    return Chem.Mol(rwmol)



# =============================================================================
# def delete_label_atoms(mol, delete_atom_symbol='Po'):
#     rwmol = Chem.RWMol(mol)
#     rwmol.UpdatePropertyCache(strict=False)
#     
#     Po_idxs = find_label_atom_idxs(rwmol, atom_symbol=delete_atom_symbol)
#     print("atline133  Po_idxs = ", Po_idxs)
# 
#     for Po_idx in Po_idxs:
#         Po_atom = rwmol.GetAtomWithIdx(Po_idx)
#         nbrs = Po_atom.GetNeighbors()
#         #print("Po_atom.GetNeighbors() are: ", Po_atom.GetNeighbors() )
#         if len(nbrs) >= 2:
#             left_atom_idx = nbrs[0].GetIdx()
#             right_atom_idx = nbrs[1].GetIdx()
#             rwmol.RemoveBond(left_atom_idx, Po_idx)
#             rwmol.RemoveBond(right_atom_idx, Po_idx)
#             rwmol.AddBond(left_atom_idx, right_atom_idx, Chem.BondType.SINGLE)
#             rwmol.RemoveAtom(Po_idx)
#         
#     return Chem.Mol(rwmol)
# =============================================================================


def exchange_dummy_and_Po_label_idx(mol, label_atom1_symbol='*', label_atom2_symbol='Po'):
    label_atom1_idx = find_label_atom_idx(mol, atom_symbol=label_atom1_symbol)
    label_atom2_idx = find_label_atom_idx(mol, atom_symbol=label_atom2_symbol)
    
    if label_atom1_symbol == '*' and label_atom2_symbol == 'Po':    
        label_atom1_AtomicNum = 0
        label_atom2_AtomicNum = 84
    else:
        label_atom1_AtomicNum = 84
        label_atom2_AtomicNum = 0
    
    #print("label_atom1_symbol, label_atom2_symbol, label_atom1_idx, label_atom2_idx, label_atom2_AtomicNum, label_atom1_AtomicNum : ")
    #print(label_atom1_symbol, label_atom2_symbol, label_atom1_idx, label_atom2_idx, label_atom1_AtomicNum, label_atom2_AtomicNum)
    
    mol.GetAtomWithIdx(label_atom1_idx).SetAtomicNum(label_atom2_AtomicNum)
    mol.GetAtomWithIdx(label_atom2_idx).SetAtomicNum(label_atom1_AtomicNum)
    Chem.SanitizeMol(mol)


def replace_dummy_by_Po_label(mol, original_label_symbol='*', final_label_symbol='Po', reverse=False):
    original_label_idx = find_label_atom_idx(mol, atom_symbol=original_label_symbol)
    
    if reverse == False:    
        replace_label_AtomicNum = 84
    else:
        replace_label_AtomicNum = 0
    
    mol.GetAtomWithIdx(original_label_idx).SetAtomicNum(replace_label_AtomicNum)
    Chem.SanitizeMol(mol)


def replace_dummy_by_Po_label_v1(mol, original_label_symbol='*', final_label_symbol='Po', reverse=False):
    mol1 = Chem.MolFromSmiles( Chem.MolToSmiles(mol) ) 
    original_label_idx = find_label_atom_idx(mol1, atom_symbol=original_label_symbol)
    
    if reverse == False:    
        replace_label_AtomicNum = 84
    else:
        replace_label_AtomicNum = 0
    
    mol1.GetAtomWithIdx(original_label_idx).SetAtomicNum(replace_label_AtomicNum)
    # Chem.SanitizeMol(mol1)    
    
    return mol1


def draw_mols(mols, nb_mols=9, file_name='./foo-gridimage-mols.png', molsPerRow=4, size=(400, 400), legends=None):
    # Compute2DCoords for dipection
    for mol in mols[:nb_mols]:
        AllChem.Compute2DCoords(mol)
    
    # View the enumerated molecules: ---------------------------------------------
    img = Draw.MolsToGridImage(
                               mols[:nb_mols], 
                               molsPerRow=molsPerRow, 
                               subImgSize=size,
                               legends=legends,
                               ) 
    img.save(file_name)
    
    # print(type(img))
    # print(dir(img))
    
    img.close()


#------------------------------------------------------------------------------
def D_Pi_A_Enumeration(cores1_smis, chains_smis, cores2_smis, verbse=False):
    cores1 = [Chem.MolFromSmiles(smi) for smi in cores1_smis]
    chains = [Chem.MolFromSmiles(smi) for smi in chains_smis] 
    # [Te] or [Po] divalent element for subsquent substitution 
    cores2 = [Chem.MolFromSmiles(smi) for smi in cores2_smis]
    
    cores1_chains = []
    for core1 in cores1:
        for chain in chains:
            core1_chain = Chem.ReplaceSubstructs(core1, Chem.MolFromSmarts('[#0]'), chain)
            cores1_chains.append(core1_chain[0])
    
    cores1_chains_cores2 = []
    
    for core2  in cores2:
        for core1_chain in cores1_chains: 
            core1_chain_core2 = Chem.ReplaceSubstructs(core1_chain, Chem.MolFromSmarts('[Po]'), core2)
            cores1_chains_cores2.append(core1_chain_core2[0])    
    
    for mol in cores1_chains_cores2:
        Chem.SanitizeMol(mol)
    
    if verbse:
        print("# info on D_Pi_A_Enumeration( )")
        print("\n# cores1 :")
        for mol in cores1:
            print(Chem.MolToSmiles(mol))         
        
        print("\n# chains :")
        for mol in chains:
            print(Chem.MolToSmiles(mol)) 
            
        print("\n# cores2 :")
        for mol in cores2:
            print(Chem.MolToSmiles(mol)) 
            
        print("\n# cores1_chains :")
        for mol in cores1_chains:
            print(Chem.MolToSmiles(mol)) 
        
        print("\nThe product cores1_chains_cores2 molecules are: ")
        print("-"*80)
        for mol in cores1_chains_cores2:
            print(Chem.MolToSmiles(mol)) 

        print("-"*80)
        print("# The lens of cores1, chains, cores2, cores1_chains, cores1_chains_cores2 are:")  
        print(len(cores1), len(chains), len(cores2), len(cores1_chains), len(cores1_chains_cores2) )  

    return delete_repeated_smiles_of_mols(cores1_chains_cores2) 


def Cores_Chains_Enumeration(cores_smis, chains_smis, verbse=False):
    cores = [Chem.MolFromSmiles(smi) for smi in cores_smis]
    chains = [Chem.MolFromSmiles(smi) for smi in chains_smis] 
    
    cores_chains = []
    for core1 in cores:
        for chain in chains:
            core1_chain = Chem.ReplaceSubstructs(core1, Chem.MolFromSmarts('[#0]'), chain)
            cores_chains.append(core1_chain[0])
    
    for mol in cores_chains:
        Chem.SanitizeMol(mol)
    
    if verbse:
        print("# info on Cores_Chains_Enumeration( )")
        print("\n# cores :")
        for mol in cores:
            print(Chem.MolToSmiles(mol))         
        
        print("\n# chains :")
        for mol in chains:
            print(Chem.MolToSmiles(mol)) 
                 
        print("\nThe product cores_chains molecules are: ")
        print("-"*80)
        for mol in cores_chains:
            print(Chem.MolToSmiles(mol)) 

        print("-"*80)
        print("# The lens of cores, chains, cores_chains are:")  
        print(len(cores), len(chains), len(cores_chains) )  

    return delete_repeated_smiles_of_mols(cores_chains) 


def Cores_Chains_Enumeration_FullSubstitute(cores_smis, chains_smis, verbse=False):
    cores = [Chem.MolFromSmiles(smi) for smi in cores_smis]
    chains = [Chem.MolFromSmiles(smi) for smi in chains_smis] 
    
    cores_chains = []
    for core in cores:
        for chain in chains:
            core_chain = Chem.ReplaceSubstructs(core, Chem.MolFromSmarts('[#0]'), chain, True)
            cores_chains.append(core_chain[0])
    
    for mol in cores_chains:
        Chem.SanitizeMol(mol)
    
    if verbse:
        print("# info on Cores_Chains_Enumeration_FullSubstitute( )")
        print("\n# cores :")
        for mol in cores:
            print(Chem.MolToSmiles(mol))         
        
        print("\n# chains :")
        for mol in chains:
            print(Chem.MolToSmiles(mol)) 
                 
        print("\nThe product core_chains molecules are: ")
        print("-"*80)
        for mol in cores_chains:
            print(Chem.MolToSmiles(mol)) 

        print("-"*80)
        print("# The lens of cores, chains, core_chains are:")  
        print(len(cores), len(chains), len(cores_chains) )  

    return delete_repeated_smiles_of_mols(cores_chains) 


#------------------------------------------------------------------------------
def D_A_Enumeration_new(ligandA_smis, ligandB_smis, verbse=False):
    '''
    This function can return the combined chemical compound library on condition that 
    Donors and Acceptors are given via args ligandA_smis and ligandB_smis, and 
    the substitution position of Donor is labeled by [*], meanwhile Acceptor by [Po].

    Parameters
    ----------
    ligandA_smis : list of smiles.
        DESCRIPTION.
    ligandB_smis : list of smiles.
        DESCRIPTION.
    verbse : logical, optional
        DESCRIPTION. The default is False.

    Returns
    -------
    DA_mols : list of rdkit mol. 
        DESCRIPTION.

    '''
    ligandAs = [Chem.MolFromSmiles(smi) for smi in ligandA_smis]
    ligandBs = [Chem.MolFromSmiles(smi) for smi in ligandB_smis]
    
    ligandAs_Po_ligandBs = []
    
    for ligandA in ligandAs:
        for ligandB in ligandBs:
            
            Po_idx = find_label_atom_idx(ligandB, atom_symbol='Po')
            
            ligandA_Po_ligandB = Chem.ReplaceSubstructs(ligandA, 
                                                        Chem.MolFromSmarts('[#0]'), 
                                                        ligandB, 
                                                        replacementConnectionPoint=Po_idx )
            ligandAs_Po_ligandBs.append(ligandA_Po_ligandB[0])
    
    DA_mols = []
    
    for mol in ligandAs_Po_ligandBs:
        mol_rev = delete_label_atom(mol, delete_atom_symbol='Po')
        DA_mols.append(mol_rev)
    

    for mol in DA_mols:
        Chem.SanitizeMol(mol)
    
    if verbse:
        print("# info on DA_Enumeration_new( )")
        print("\n# ligandAs :")
        for mol in ligandAs:
            print(Chem.MolToSmiles(mol))         
        
        print("\n# ligandBs :")
        for mol in ligandBs:
            print(Chem.MolToSmiles(mol))
                 
        print("\n#The product D_A molecules are: ")
        print("-"*80)
        for mol in DA_mols:
            print(Chem.MolToSmiles(mol)) 

        print("-"*80)
        print("# The lens of ligandAs, ligandBs, DA_mols are:")  
        print(len(ligandAs), len(ligandBs), len(DA_mols) )  

    return delete_repeated_smiles_of_mols(DA_mols)


def D_A_Enumeration_new_FullSubstitute(ligandA_smis, ligandB_smis, verbse=False):
    '''
    This function can return the combined chemical compound library on condition that 
    Donors and Acceptors are given via args ligandA_smis and ligandB_smis, and 
    the substitution position of Donor is labeled by [*], meanwhile Acceptor by [Po].

    Parameters
    ----------
    ligandA_smis : list of smiles.
        DESCRIPTION.
    ligandB_smis : list of smiles.
        DESCRIPTION.
    verbse : logical, optional
        DESCRIPTION. The default is False.

    Returns
    -------
    DA_mols : list of rdkit mol. 
        DESCRIPTION.

    '''
    ligandAs = [Chem.MolFromSmiles(smi) for smi in ligandA_smis]
    ligandBs = [Chem.MolFromSmiles(smi) for smi in ligandB_smis]
    
    ligandAs_Po_ligandBs = []
    
    for ligandA in ligandAs:
        for ligandB in ligandBs:
            
            Po_idx = find_label_atom_idx(ligandB, atom_symbol='Po')
            
            ligandA_Po_ligandB = Chem.ReplaceSubstructs(ligandA, 
                                                        Chem.MolFromSmarts('[#0]'), 
                                                        ligandB, 
                                                        replaceAll=True,
                                                        replacementConnectionPoint=Po_idx )
            ligandAs_Po_ligandBs.append(ligandA_Po_ligandB[0])
    
    DA_mols = []
    
    for mol in ligandAs_Po_ligandBs:
        #mol_rev = delete_label_atoms(mol, delete_atom_symbol='Po')
        mol_rev = delete_label_atoms_new(mol, delete_atom_symbol='Po')
        DA_mols.append(mol_rev)
    

    for mol in DA_mols:
        Chem.SanitizeMol(mol)
    
    if verbse:
        print("# info on DA_Enumeration_new( )")
        print("\n# ligandAs :")
        for mol in ligandAs:
            print(Chem.MolToSmiles(mol))         
        
        print("\n# ligandBs :")
        for mol in ligandBs:
            print(Chem.MolToSmiles(mol))
                 
        print("\n#The product D_A molecules are: ")
        print("-"*80)
        for mol in DA_mols:
            print(Chem.MolToSmiles(mol)) 

        print("-"*80)
        print("# The lens of ligandAs, ligandBs, DA_mols are:")  
        print(len(ligandAs), len(ligandBs), len(DA_mols) )  

    return delete_repeated_smiles_of_mols(DA_mols)





def D_Pi_A_Enumeration_new(ligandA_smis, pi_bridge_smis, ligandB_smis, verbse=False):
    '''
    This function can return the combined chemical compound library on condition that 
    Donors, Pi_bridges and Acceptors are given via args ligandA_smis, pi_bridge_smis and ligandB_smis, 
    and the substitution position of 
    Donor is labeled by [*], 
    Pi_bridge by [Po]--pi-bridge--[*],
    meanwhile Acceptor by [Po].

    Parameters
    ----------
    ligandA_smis : list of smiles.
        DESCRIPTION.
    pi_bridge_smis : list of smiles.
        DESCRIPTION.
    ligandB_smis : list of smiles.
        DESCRIPTION.
    verbse : logical, optional
        DESCRIPTION. The default is False.

    Returns
    -------
    DPiA_mols : list of rdkit mol. D_A_Enumeration_new
        DESCRIPTION.

    '''
    ligandAs = [Chem.MolFromSmiles(smi) for smi in ligandA_smis]
    pi_bridges = [Chem.MolFromSmiles(smi) for smi in pi_bridge_smis]
    ligandBs = [Chem.MolFromSmiles(smi) for smi in ligandB_smis]
    
    #---- create ligandAs_pi_bridges provided ligandAs ('*') and pi_bridges ('Po'---'*'), 
    #-----ligandAs_pi_bridges also have label atom [*] for further substitution
    ligandAs_Po_pi_bridges = []
    
    for ligandA in ligandAs:
        for pi_bridge in pi_bridges:
            Po_idx = find_label_atom_idx(pi_bridge, atom_symbol='Po')
            
            ligandA_Po_pi_bridge = Chem.ReplaceSubstructs(ligandA, 
                                                        Chem.MolFromSmarts('[#0]'), 
                                                        pi_bridge, 
                                                        replacementConnectionPoint=Po_idx )
            ligandAs_Po_pi_bridges.append(ligandA_Po_pi_bridge[0])
    
    ligandAs_pi_bridges = []
    
    for mol in ligandAs_Po_pi_bridges:
        mol_rev = delete_label_atom(mol, delete_atom_symbol='Po')
        ligandAs_pi_bridges.append(mol_rev)
    
    #---- create DPiA_mols provided ligandAs_Po_pi_bridges ('*') and ligandBs ('Po')
    ligandAs_pi_bridges_Po_ligandBs = []
    
    for ligandA_pi_bridge in ligandAs_pi_bridges:
        for ligandB in ligandBs:
            Po_idx = find_label_atom_idx(ligandB, atom_symbol='Po')
            
            ligandA_pi_bridge_Po_ligandB = Chem.ReplaceSubstructs(ligandA_pi_bridge, 
                                                        Chem.MolFromSmarts('[#0]'), 
                                                        ligandB, 
                                                        replacementConnectionPoint=Po_idx )
            ligandAs_pi_bridges_Po_ligandBs.append(ligandA_pi_bridge_Po_ligandB[0])
    
    DPiA_mols = []
    
    for mol in ligandAs_pi_bridges_Po_ligandBs:
        mol_rev = delete_label_atom(mol, delete_atom_symbol='Po')
        DPiA_mols.append(mol_rev)
    
    #----- Sanitize mols ------
    for mol in DPiA_mols:
        Chem.SanitizeMol(mol)
    
    if verbse:
        print("# info on D_Pi_A_Enumeration_new( )")
        print("\n# ligandAs :")
        for mol in ligandAs:
            print(Chem.MolToSmiles(mol))      
            
        print("\n# pi_bridges :")
        for mol in pi_bridges:
            print(Chem.MolToSmiles(mol))
        
        print("\n# ligandBs :")
        for mol in ligandBs:
            print(Chem.MolToSmiles(mol))
                 
        print("\n#The product D_Pi_A molecules are: ")
        print("-"*80)
        for mol in DPiA_mols:
            print(Chem.MolToSmiles(mol)) 

        print("-"*80)
        print("# The lens of ligandAs, ligandBs, pi_bridges, DPiA_mols are:")  
        print(len(ligandAs), len(ligandBs), len(pi_bridges), len(DPiA_mols) )  

    return delete_repeated_smiles_of_mols(DPiA_mols) 


def D_A_D_Enumeration_new(donor_smis, acceptorAsPiBridge_smis, verbse=False):
    '''
    This function can return the combined chemical compound library on condition that 
    Donors, acceptorAsPiBridge are given via args donor_smis, acceptorAsPiBridge_smis, 
    and the substitution position of 
    1st Donor is labeled by [*], 
    Pi_bridge by [Po]--pi-bridge--[*]

    Parameters
    ----------
    donor_smis : list of smiles.
        DESCRIPTION.
    acceptorAsPiBridge_smis : list of smiles.
        DESCRIPTION.
    verbse : logical, optional
        DESCRIPTION. The default is False.

    Returns
    -------
    DPiA_mols : list of rdkit mol. 
        DESCRIPTION.

    '''
    
    donors_2 = [ Chem.MolFromSmiles(smi) for smi in donor_smis ]
    for donor in donors_2:
        replace_dummy_by_Po_label(donor, original_label_symbol='*', final_label_symbol='Po')
        
    donor_2_smis = [Chem.MolToSmiles(mol)  for mol in donors_2]    
    DAD_mols = D_Pi_A_Enumeration_new(donor_smis, acceptorAsPiBridge_smis, donor_2_smis, verbse=verbse)
    
    return DAD_mols


def D_Pi_A_Enumeration_new_exchange_pibridge_dummy_po(ligandA_smis, pi_bridge_smis, ligandB_smis, verbse=False):
    pi_bridges_exchange = [Chem.MolFromSmiles(smi) for smi in pi_bridge_smis]
    for mol in pi_bridges_exchange:
        exchange_dummy_and_Po_label_idx(mol, label_atom1_symbol='*', label_atom2_symbol='Po')
    pi_bridge_smis_exchange = [ Chem.MolToSmiles(mol)  for mol in  pi_bridges_exchange]
    
    return D_Pi_A_Enumeration_new(ligandA_smis, pi_bridge_smis_exchange, ligandB_smis, verbse=verbse)


def D_A_D_Enumeration_new_exchange_acceptoraspibridge_dummy_po(donor_smis, acceptorAsPiBridge_smis, verbse=False):
    acceptorAsPiBridges_exchange = [Chem.MolFromSmiles(smi) for smi in acceptorAsPiBridge_smis]
    for mol in acceptorAsPiBridges_exchange:
        exchange_dummy_and_Po_label_idx(mol, label_atom1_symbol='*', label_atom2_symbol='Po')
    acceptorAsPiBridge_smis_exchange = [ Chem.MolToSmiles(mol)  for mol in  acceptorAsPiBridges_exchange]
    
    return D_A_D_Enumeration_new(donor_smis, acceptorAsPiBridge_smis_exchange, verbse=verbse)


def D3_A_Enumeration_new(ligandA_smis, ligandB_smis, verbse=False):
    '''
    This function can return the combined chemical compound library on condition that 
    Donors and Acceptors are given via args ligandA_smis and ligandB_smis, and 
    the substitution position of Donor is labeled by [*], meanwhile Acceptor by three [Po].

    Parameters
    ----------
    ligandA_smis : list of smiles.
        DESCRIPTION.
    ligandB_smis : list of smiles.
        DESCRIPTION.
    verbse : logical, optional
        DESCRIPTION. The default is False.

    Returns
    -------
    D3A_mols : list of rdkit mol. 
        DESCRIPTION.

    '''
    ligandAs = [Chem.MolFromSmiles(smi) for smi in ligandA_smis]
    ligandBs = [Chem.MolFromSmiles(smi) for smi in ligandB_smis]
    
    D3_dummy_A_mols = []
    for ligandB in ligandBs:
        for ligandA in ligandAs:
            dummy_idx = find_label_atom_idx(ligandA, atom_symbol='*')
            D3_dummy_A = Chem.ReplaceSubstructs(ligandB, 
                                                Chem.MolFromSmarts('[Po]'), 
                                                ligandA, 
                                                replaceAll=True,
                                                replacementConnectionPoint=dummy_idx, 
                                                )
        
            D3_dummy_A_mols.append(D3_dummy_A[0])
    
    D3A_mols = []
    
    for mol in D3_dummy_A_mols:
        mol_rev1 = delete_label_atom(mol, delete_atom_symbol='*')
        mol_rev2 = delete_label_atom(mol_rev1, delete_atom_symbol='*')
        mol_rev3 = delete_label_atom(mol_rev2, delete_atom_symbol='*')
        D3A_mols.append(mol_rev3)
    

    for mol in D3A_mols:
        Chem.SanitizeMol(mol)
    
    if verbse:
        print("# info on D3_A_Enumeration_new( )")
        print("\n# ligandAs :")
        for mol in ligandAs:
            print(Chem.MolToSmiles(mol))         
        
        print("\n# ligandBs :")
        for mol in ligandBs:
            print(Chem.MolToSmiles(mol))
                 
        print("\n#The product D3_A molecules are: ")
        print("-"*80)
        #print("len(D3A_mols) is ", len(D3A_mols))
        for mol in D3A_mols:
            print(Chem.MolToSmiles(mol)) 

        print("-"*80)
        print("# The lens of ligandAs, ligandBs, D3A_mols are:")  
        print(len(ligandAs), len(ligandBs), len(D3A_mols) )  

    return delete_repeated_smiles_of_mols(D3A_mols)



def D_Pi_A_Pi_D_Enumeration_new_symmetric(ligandA_smis, pi_bridge_smis, ligandB_smis, verbse=False):
    '''
    This function can return the combined chemical compound library on condition that 
    Donors, Pi_bridges and Acceptors are given via args ligandA_smis, pi_bridge_smis and ligandB_smis, 
    and the substitution position of 
    Donor is labeled by [*], 
    Pi_bridge by [Po]--pi-bridge--[*],
    meanwhile Acceptor by [Po]--acceptor--[*].

    Parameters
    ----------
    ligandA_smis : list of smiles.
        DESCRIPTION.
    pi_bridge_smis : list of smiles.
        DESCRIPTION.
    ligandB_smis : list of smiles.
        DESCRIPTION.
    verbse : logical, optional
        DESCRIPTION. The default is False.

    Returns
    -------
    DPiAPiD_mols : list of rdkit mol. 
        DESCRIPTION.

    '''
    ligandAs = [Chem.MolFromSmiles(smi) for smi in ligandA_smis]
    pi_bridges = [Chem.MolFromSmiles(smi) for smi in pi_bridge_smis]
    ligandBs = [Chem.MolFromSmiles(smi) for smi in ligandB_smis]
    
    #---- create ligandAs_pi_bridges provided ligandAs ('*') and pi_bridges ('Po'---'*'), 
    #-----ligandAs_pi_bridges also have label atom [*] for further substitution
    ligandAs_Po_pi_bridges = []
    ligandAs_pi_bridges = []
    ligandAs_pi_bridges_Po_ligandBs = []
    ligandAs_pi_bridges_ligandBs = []
    ligandAs_pi_bridges_ligandBs_Po_pi_bridges = []
    ligandAs_pi_bridges_ligandBs_pi_bridges = []
    ligandAs_pi_bridges_ligandBs_pi_bridges_Po_ligandAs = [] 
    ligandAs_pi_bridges_ligandBs_pi_bridges_ligandAs = []
    
    
    for ligandA in ligandAs:
        for pi_bridge in pi_bridges:
            for ligandB in ligandBs:
                Po_idx = find_label_atom_idx(pi_bridge, atom_symbol='Po')
                
                ligandA_Po_pi_bridge = Chem.ReplaceSubstructs(ligandA, 
                                                            Chem.MolFromSmarts('[#0]'), 
                                                            pi_bridge, 
                                                            replacementConnectionPoint=Po_idx )
                ligandAs_Po_pi_bridges.append(ligandA_Po_pi_bridge[0])
                
                ligandA_pi_bridge = delete_label_atom(ligandA_Po_pi_bridge[0], 
                                                      delete_atom_symbol='Po')
                
                ligandAs_pi_bridges.append(ligandA_pi_bridge)
                
                # print("DPi", Chem.MolToSmiles(ligandA_pi_bridge) )
                

                Po_idx1 = find_label_atom_idx(ligandB, atom_symbol='Po')
                
                ligandA_pi_bridge_Po_ligandB = Chem.ReplaceSubstructs(ligandA_pi_bridge, 
                                                                        Chem.MolFromSmarts('[#0]'), 
                                                                        ligandB, 
                                                                        replacementConnectionPoint=Po_idx1 )
                ligandAs_pi_bridges_Po_ligandBs.append(ligandA_pi_bridge_Po_ligandB[0])
                
                ligandA_pi_bridge_ligandB = delete_label_atom(ligandA_pi_bridge_Po_ligandB[0], 
                                                              delete_atom_symbol='Po')
            
                ligandAs_pi_bridges_ligandBs.append(ligandA_pi_bridge_ligandB)
                
                # print("DPiA", Chem.MolToSmiles(ligandA_pi_bridge_ligandB) )
                
                #---------------------------------------------------------------
                # Po_idx2 = find_label_atom_idx(pi_bridge, atom_symbol='Po')
                
                ligandA_pi_bridge_ligandB_Po_pi_bridge = Chem.ReplaceSubstructs(ligandA_pi_bridge_ligandB, 
                                                                                Chem.MolFromSmarts('[#0]'), 
                                                                                pi_bridge, 
                                                                                replacementConnectionPoint=Po_idx )
                ligandAs_pi_bridges_ligandBs_Po_pi_bridges.append(ligandA_pi_bridge_ligandB_Po_pi_bridge[0])
                
                ligandA_pi_bridge_ligandB_pi_bridge = delete_label_atom(ligandA_pi_bridge_ligandB_Po_pi_bridge[0], 
                                                                        delete_atom_symbol='Po')
                
                ligandAs_pi_bridges_ligandBs_pi_bridges.append(ligandA_pi_bridge_ligandB_pi_bridge)

                # print("DPiAPi", Chem.MolToSmiles(ligandA_pi_bridge_ligandB_pi_bridge) )                                
    

                ligandA_rev = replace_dummy_by_Po_label_v1(ligandA, 
                                                           original_label_symbol='*', 
                                                           final_label_symbol='Po', 
                                                           reverse=False)
                
                Po_idx3 = find_label_atom_idx(ligandA_rev, atom_symbol='Po')
                
                ligandA_pi_bridge_ligandB_pi_bridge_Po_ligandA = Chem.ReplaceSubstructs(ligandA_pi_bridge_ligandB_pi_bridge, 
                                                                                        Chem.MolFromSmarts('[#0]'), 
                                                                                        ligandA_rev, 
                                                                                        replacementConnectionPoint=Po_idx3 )
                ligandAs_pi_bridges_ligandBs_pi_bridges_Po_ligandAs.append(ligandA_pi_bridge_ligandB_pi_bridge_Po_ligandA[0])
                
                ligandA_pi_bridge_ligandB_pi_bridge_ligandA = delete_label_atom(ligandA_pi_bridge_ligandB_pi_bridge_Po_ligandA[0], 
                                                                                delete_atom_symbol='Po')
                
                ligandAs_pi_bridges_ligandBs_pi_bridges_ligandAs.append(ligandA_pi_bridge_ligandB_pi_bridge_ligandA)
                
                print("DPiAPiD", Chem.MolToSmiles(ligandA_pi_bridge_ligandB_pi_bridge_ligandA) )
    
    
    DPiAPiD_mols = ligandAs_pi_bridges_ligandBs_pi_bridges_ligandAs
    
    
    #----- Sanitize mols ------
    for mol, i in zip(DPiAPiD_mols, range(len(DPiAPiD_mols))):
        try:
            Chem.SanitizeMol(mol)
        except: 
            print("sanitizemol failed: ")
            print(Chem.MolToSmiles(mol))
    
    if verbse:
        print("# info on D_Pi_A_Pi_D_Enumeration_new_symmetric( )")
        print("\n# ligandAs :")
        for mol in ligandAs:
            print(Chem.MolToSmiles(mol))      
            
        print("\n# pi_bridges :")
        for mol in pi_bridges:
            print(Chem.MolToSmiles(mol))
        
        print("\n# ligandBs :")
        for mol in ligandBs:
            print(Chem.MolToSmiles(mol))
                 
        print("\n#The product D_Pi_A_Pi_D molecules are: ")
        print("-"*80)
        for mol in DPiAPiD_mols:
            print(Chem.MolToSmiles(mol)) 

        print("-"*80)
        print("# The lens of ligandAs, ligandBs, pi_bridges, DPiAPiD_mols are:")  
        print(len(ligandAs), len(ligandBs), len(pi_bridges), len(DPiAPiD_mols) )  

    return delete_repeated_smiles_of_mols(DPiAPiD_mols) 



def D_A_D_Enumeration_new_symmetric(ligandA_smis, ligandB_smis, verbse=False):
    '''
    This function can return the combined chemical compound library on condition that 
    Donors and Acceptors are given via args ligandA_smis and ligandB_smis, 
    and the substitution position of 
    Donor is labeled by [*], 
    meanwhile Acceptor by [Po]--acceptor--[*].

    Parameters
    ----------
    ligandA_smis : list of smiles.
        DESCRIPTION.
    ligandB_smis : list of smiles.
        DESCRIPTION.
    verbse : logical, optional
        DESCRIPTION. The default is False.

    Returns
    -------
    DAD_mols : list of rdkit mol. 
        DESCRIPTION.

    '''
    ligandAs = [Chem.MolFromSmiles(smi) for smi in ligandA_smis]
    ligandBs = [Chem.MolFromSmiles(smi) for smi in ligandB_smis]
    
    #---- create ligandAs_ligandBs provided ligandAs ('*') and ligandAs ('Po'---'*'), 
    #-----ligandAs_ligandBs also have label atom [*] for further substitution

    ligandAs_Po_ligandBs = []
    ligandAs_ligandBs = []
    ligandAs_ligandBs_Po_ligandAs = [] 
    ligandAs_ligandBs_ligandAs = []
    
    
    for ligandA in ligandAs:
            for ligandB in ligandBs:
                Po_idx = find_label_atom_idx(ligandB, atom_symbol='Po')
                
                ligandA_Po_ligandB = Chem.ReplaceSubstructs(ligandA, 
                                                            Chem.MolFromSmarts('[#0]'), 
                                                            ligandB, 
                                                            replacementConnectionPoint=Po_idx )
                ligandAs_Po_ligandBs.append(ligandA_Po_ligandB[0])
                
                ligandA_ligandB = delete_label_atom(ligandA_Po_ligandB[0], 
                                                      delete_atom_symbol='Po')
                
                ligandAs_ligandBs.append(ligandA_ligandB)
                
                # print("DA", Chem.MolToSmiles(ligandA_ligandB) )
                
                #---------------------------------------------------------------                             
                ligandA_rev = replace_dummy_by_Po_label_v1(ligandA, 
                                                           original_label_symbol='*', 
                                                           final_label_symbol='Po', 
                                                           reverse=False)
                
                Po_idx3 = find_label_atom_idx(ligandA_rev, atom_symbol='Po')
                
                ligandA_ligandB_Po_ligandA = Chem.ReplaceSubstructs(ligandA_ligandB, 
                                                                    Chem.MolFromSmarts('[#0]'), 
                                                                    ligandA_rev, 
                                                                    replacementConnectionPoint=Po_idx3 )
                ligandAs_ligandBs_Po_ligandAs.append(ligandA_ligandB_Po_ligandA[0])
                
                ligandA_ligandB_ligandA = delete_label_atom(ligandA_ligandB_Po_ligandA[0], 
                                                            delete_atom_symbol='Po')
                
                ligandAs_ligandBs_ligandAs.append(ligandA_ligandB_ligandA)
                
                print("DAD", Chem.MolToSmiles(ligandA_ligandB_ligandA) )
    
    
    DAD_mols = ligandAs_ligandBs_ligandAs
    
    
    #----- Sanitize mols ------
    for mol, i in zip(DAD_mols, range(len(DAD_mols))):
        try:
            Chem.SanitizeMol(mol)
        except: 
            print("sanitizemol failed: ")
            print(Chem.MolToSmiles(mol))
    
    if verbse:
        print("# info on D_A_D_Enumeration_new_symmetric( )")
        print("\n# ligandAs :")
        for mol in ligandAs:
            print(Chem.MolToSmiles(mol))      
        
        print("\n# ligandBs :")
        for mol in ligandBs:
            print(Chem.MolToSmiles(mol))
                 
        print("\n#The product D_A_D molecules are: ")
        print("-"*80)
        for mol in DAD_mols:
            print(Chem.MolToSmiles(mol)) 

        print("-"*80)
        print("# The lens of ligandAs, ligandBs, DAD_mols are:")  
        print(len(ligandAs), len(ligandBs), len(DAD_mols) )  

    return delete_repeated_smiles_of_mols(DAD_mols) 



#-----------------------------------------------------------------------------
def try_embedmolecule(mol):
    mol1 = Chem.AddHs(mol)
    conf_idx = AllChem.EmbedMolecule(mol1,
                                     #maxAttempts=0,
                                     randomSeed=38,
                                     )
                                     
    nb_try_EmbedMolecule = 1
    while conf_idx == -1 :
        print("nb_try_EmbedMolecule is ", nb_try_EmbedMolecule)
        print("AllChem.EmbedMolecule failed!")
        print("conf_idx is ", conf_idx)
        print("reset randomSeed and try")
        seednum = np.random.randint(1, 100000)        
        conf_idx = AllChem.EmbedMolecule(mol1,
                                 #maxAttempts=0,
                                 randomSeed=seednum,
                                 )
        nb_try_EmbedMolecule += 1
    
    print("The final conf_idx is ", conf_idx)
    
    print("nb_try_EmbedMolecule = ", nb_try_EmbedMolecule)
    
    return mol1, nb_try_EmbedMolecule


#-----------------------------------------------------------------------------
def write_mol_to_sdf(mol, cpd_name, outfile_path='./project/G0000/separate_mols/'):
    
    outname=  outfile_path + '%s.sdf'%(cpd_name)
    #print(outname)
    one_w = Chem.SDWriter(outname)
    m_temp = Chem.AddHs(mol)
    m1_temp, nb_try_EmbedMolecule = try_embedmolecule(m_temp)
    m1_temp.SetProp("_Name", cpd_name)
    one_w.write(m1_temp)


def write_mols_paralell(mols, cpd_names, outfile_path='./project/G0000/separate_mols/'):
    #-------  paralell write mols-----------------------------------------------------------
    print("The outfile_path is : %s" % outfile_path )
    
    start = time.time()
    
    mkdir_separate_mols = 'mkdir   ' + outfile_path + '  2>/dev/null  '
    (status, output) = subprocess.getstatusoutput( mkdir_separate_mols )
    
    rm_GXXXX_separate_mols_sdf = 'rm -f   '  +  outfile_path + 'cpd-*.sdf  '
    (status0, output0) = subprocess.getstatusoutput( rm_GXXXX_separate_mols_sdf )
    
    Parallel(n_jobs=-1)(delayed(write_mol_to_sdf)(mol, cpd_name, outfile_path) \
                        for mol, cpd_name in zip( tqdm(mols), cpd_names) )
    
    end = time.time()
    
    print('-'*60)
    print("write_mol_to_sdf run in paralell.")
    print('{:.4f} s'.format(end-start))


#-----------------------------------------------------------------------------
def write_mol_to_sdf_notEmbed(mol, 
                              cpd_name, 
                              outfile_path='./project/G0000/separate_mols/'):
    
    outname=  outfile_path + '%s.sdf'%(cpd_name)
    #print(outname)
    one_w = Chem.SDWriter(outname)
    m_temp = mol
    
    m_temp.SetProp("_Name", cpd_name)
    
    # if m_temp.GetProp("_Name") is None: 
    #     m_temp.SetProp("_Name", cpd_name)
    # else:
    #     print(m_temp.GetProp("_Name"))
    
    one_w.write(m_temp)
    
    with open(outname, 'a+') as fh:
        fh.write("$$$$" +"\n")


def write_mols_paralell_notEmbed(mols, 
                                 cpd_names, 
                                 outfile_path='./project/G0000/separate_mols/'):
    #-------  paralell write mols-----------------------------------------------------------
    print("The outfile_path is : %s" % outfile_path )
    
    start = time.time()
    
    rm_GXXXX_separate_mols_sdf = 'rm -f   '  +  outfile_path + '*cpd*.sdf  '
    (status0, output0) = subprocess.getstatusoutput( rm_GXXXX_separate_mols_sdf )
    
    Parallel(n_jobs=-1)(delayed(write_mol_to_sdf_notEmbed)(mol, cpd_name, outfile_path) \
                        for mol, cpd_name in zip( tqdm(mols), cpd_names) )
    
    end = time.time()
    
    print('-'*60)
    print("write_mol_to_sdf run in paralell without embedmolecule.")
    print('{:.4f} s'.format(end-start))



#_________________________________________________________________________________
#*********************************************************************************
#-----__main__ function begins here ----------------------------------------------
if __name__ == '__main__':

    print("Begins")
    print("-"*80)
    
# =============================================================================
#     #---- test on D_Pi_A_Enumeration( )    
#     cores1_smis = ['n1c(*)c[nH]c1','n1cc(*)[nH]c1', 'n1cc[nH]c1(*)', 'n1ccn(*)c1']
#     chains_smis = ['C[Po]', 'CC[Po]', 'CCC[Po]'] 
#     cores2_smis = ['c1ncccc1C(=O)O', 'c1cnccc1C(=O)O', 'c1ccncc1C(=O)O']
#     D_Pi_A_mols = D_Pi_A_Enumeration(cores1_smis, chains_smis, cores2_smis, verbse=True)
#     draw_mols(D_Pi_A_mols, 16, './images/foo025--D-Pi-A-mols.png', molsPerRow=4, size=(400, 400))
#     
#     #---- test on Cores_Chains_Enumeration( )
#     cores_smis = ['n1c(*)c[nH]c1','n1cc(*)[nH]c1', 'n1cc[nH]c1(*)', 'n1ccn(*)c1']
#     chains_smis = ['CN', 'CCN', 'CCCN'] 
#     cores_chains_mols = Cores_Chains_Enumeration(cores_smis, chains_smis, verbse=True)
#     draw_mols(cores_chains_mols, 9, './images/foo025--cores-chains-mols.png', molsPerRow=3, size=(400, 400))
#     
#     #---- test on Cores_Chains_Enumeration_FullSubstitute( )
#     cores_smis = ['*[n+]1c(*)c[nH]c1(*)', 'n1cc(*)[nH]c1(*)']
#     chains_smis = ['CN', 'CCN', 'CCCN'] 
#     cores_chains_mols = Cores_Chains_Enumeration_FullSubstitute(cores_smis, chains_smis, verbse=True)
#     draw_mols(cores_chains_mols, 9, './images/foo025--cores-chains-fullsubs-mols.png', molsPerRow=3, size=(400, 400))
#
# =============================================================================     

# =============================================================================
#     #---- test on D_A_Enumeration_new()
#     ligandA_smis = ['C1=CC=CC2=C1[N](C3=C2C=CC=C3)[*]', \
#                     'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C)[*])C', \
#                     'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C(C)(C)C)[*])C(C)(C)C']
#     ligandB_smis = ['C1=CC=CC=C1[S](C2=CC=C(C=C2)[Po])(=O)=O', \
#                     'C1=CC=CC=C1C(C2=CC=C(C=C2)[Po])=O', \
#                     'C1=CC(=CC=C1C(C2=CC=C(C=C2)[Po])=O)C(C3=CC=CC=C3)=O'] 
#     
#     DA_mols = D_A_Enumeration_new(ligandA_smis, ligandB_smis, verbse=True)
#     
#     draw_mols(DA_mols, 9, './images/foo001--DA-mols-new.png', molsPerRow=3, size=(400, 400))
#     
# 
#     #---- test on D_Pi_A_Enumeration_new()
#     ligandA_smis = ['C1=CC=CC2=C1[N](C3=C2C=CC=C3)[*]', \
#                     'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C)[*])C', \
#                     'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C(C)(C)C)[*])C(C)(C)C']
#         
#     pi_bridge_smis = ['C1=CC(=CC=C1[Po])[*]', \
#                       'C1=C(C=CC3=C1C(C2=CC(=CC=C2C3(C)C)[*])(C)C)[Po]']
#         
#     ligandB_smis = ['C1=CC=CC=C1[S](C2=CC=C(C=C2)[Po])(=O)=O', \
#                     'C1=CC=CC=C1C(C2=CC=C(C=C2)[Po])=O', \
#                     'C1=CC(=CC=C1C(C2=CC=C(C=C2)[Po])=O)C(C3=CC=CC=C3)=O'] 
#     
#     DPiA_mols = D_Pi_A_Enumeration_new(ligandA_smis, pi_bridge_smis, ligandB_smis, verbse=True) 
#     
#     draw_mols(DPiA_mols, len(DPiA_mols), './images/foo001--DPiA-mols-new.png', molsPerRow=3, size=(400, 400))
# 
# =============================================================================

# =============================================================================
#     #---- test on D_A_D_Enumeration_new(donor_smis, acceptorAsPiBridge_smis, verbse=False)
#     donor_smis = ['C1=CC=CC2=C1[N](C3=C2C=CC=C3)[*]', \
#               'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C)[*])C', \
#               'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C(C)(C)C)[*])C(C)(C)C']
#     
#     acceptorAsPiBridge_smis = ['C1=CC(=CC=C1[S](C2=CC=C(C=C2)[Po])(=O)=O)[*]', \
#                             'C1=CC(=CC=C1C(C2=CC=C(C=C2)[Po])=O)C(C3=CC=C(C=C3)[*])=O',  \
#                             'C1=C(C=C(C=C1C#N)[*])[Po]']
#     
#     DAD_mols = D_A_D_Enumeration_new(donor_smis, acceptorAsPiBridge_smis, verbse=True)
# 
#     draw_mols(DAD_mols , 
#               len(DAD_mols), 
#               './images/foo001--DAD_mols-new.png', 
#               molsPerRow=3, 
#               size=(400, 400))
# =============================================================================

# =============================================================================
#     #--- test on replace_dummy_by_Po_label()
#     
#     donors = [ Chem.MolFromSmiles(smi) for smi in donor_smis ]
#     
#     for mol in donors:
#         print(Chem.MolToSmiles(mol))
# 
#     for donor in donors:
#         replace_dummy_by_Po_label(donor, original_label_symbol='*', final_label_symbol='Po')
#     
#     for mol in donors:
#         print(Chem.MolToSmiles(mol))
# =============================================================================
        
# =============================================================================
#     #----- test on exchange_dummy_and_Po_label_idx() 
#     mol = Chem.MolFromSmiles('C1=C(CCCCO)C(=CC=C1[Po])[*]')
#     molold = Chem.MolFromSmiles('C1=C(CCCCO)C(=CC=C1[Po])[*]')
#     exchange_dummy_and_Po_label_idx(mol, label_atom1_symbol='*', label_atom2_symbol='Po')
#     
#     draw_mols([molold, mol], 2, './foo001--exchange-dummy-Po.png', molsPerRow=2, size=(600, 600))
# =============================================================================
#
# =============================================================================
#     atom_idxs = find_label_atom_idxs(Chem.MolFromSmiles('C1(=NC(=NC(=N1)[Po])[Po])[Po]'), atom_symbol='Po')
#     print('atom_idxs = ', atom_idxs )
#     
#     dummy_atom_idx = find_label_atom_idx(Chem.MolFromSmiles('C1(=NC(=NC(=N1)[Po])[Po])[Po]'), atom_symbol='Po')
#     print('dummy_atom_idxs = ', dummy_atom_idx )
# =============================================================================

# =============================================================================
#     #---- test on D3_A_Enumeration_new(donor_smis, acceptor_3Po_smis, verbse=False)
#     donor_smis = ['C1=CC=CC2=C1[N](C3=C2C=CC=C3)[*]', \
#               'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C)[*])C', \
#               'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C(C)(C)C)[*])C(C)(C)C']
#     
#     acceptor_3Po_smis = ['C1(=NC(=NC(=N1)[Po])[Po])[Po]', \
#                          'C1(=NC(=CC(=N1)[Po])[Po])[Po]',  \
#                              'C1(=NC(=C(C=N1)[Po])[Po])[Po]']
#     
#     D3A_mols = D3_A_Enumeration_new(donor_smis, acceptor_3Po_smis, verbse=True)
# 
#     draw_mols(D3A_mols , 
#               len(D3A_mols), 
#               './images/foo001--D3A_mols-new.png', 
#               molsPerRow=3, 
#               size=(400, 400))
# =============================================================================

    # #---- test on D_Pi_A_Pi_D_Enumeration_new_symmetric()
    # ligandA_smis = ['C1=CC=CC2=C1[N](C3=C2C=CC=C3)[*]', \
    #                 'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C)[*])C', \
    #                 'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C(C)(C)C)[*])C(C)(C)C'
    #                 ]
        
    # pi_bridge_smis = ['C1=CC(=CC=C1[Po])[*]', \
    #                   'C1=C(C=CC3=C1C(C2=CC(=CC=C2C3(C)C)[*])(C)C)[Po]']
        
    # ligandB_smis = ['C1=CC(=CC=C1[S](C2=CC=C(C=C2)[Po])(=O)=O)[*]', \
    #                 'C1=CC(=CC=C1C(C2=CC=C(C=C2)[Po])=O)[*]', \
    #                 'C1=CC(=CC=C1C(C2=CC=C(C=C2)[Po])=O)C(C3=CC=C(C=C3)[*])=O'
    #                 ] 
    
    # DPiAPiD_mols = D_Pi_A_Pi_D_Enumeration_new_symmetric(ligandA_smis, 
    #                                                      pi_bridge_smis, 
    #                                                      ligandB_smis, 
    #                                                      verbse=True) 
    
    # draw_mols(DPiAPiD_mols, len(DPiAPiD_mols), './foo001--DPiAPiD-mols-new.png', molsPerRow=3, size=(800, 800))


    # #---- test on D_A_D_Enumeration_new_symmetric()
    # ligandA_smis = ['C1=CC=CC2=C1[N](C3=C2C=CC=C3)[*]', \
    #                 'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C)[*])C', \
    #                 'C1=CC(=CC2=C1[N](C3=C2C=C(C=C3)C(C)(C)C)[*])C(C)(C)C'
    #                 ]
        
    # ligandB_smis = ['C1=CC(=CC=C1[S](C2=CC=C(C=C2)[Po])(=O)=O)[*]', \
    #                 'C1=CC(=CC=C1C(C2=CC=C(C=C2)[Po])=O)[*]', \
    #                 'C1=CC(=CC=C1C(C2=CC=C(C=C2)[Po])=O)C(C3=CC=C(C=C3)[*])=O'
    #                 ] 
    
    # DAD_mols = D_A_D_Enumeration_new_symmetric(ligandA_smis, 
    #                                            ligandB_smis, 
    #                                            verbse=True) 
    
    # draw_mols(DAD_mols, len(DAD_mols), './foo001--DAD-mols-new-symmetric.png', molsPerRow=3, size=(800, 800))

    
    print("\nFinished!")
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar  1 13:53:00 2021

Using scikit-learn with RDKit

@author: tucy
"""


from salam.module_MD.D_Pi_A_Enumeration import draw_mols
from rdkit import Chem
#from rdkit.Chem import AllChem

from tqdm import tqdm
from salam.module_MD.D_Pi_A_Enumeration import try_embedmolecule
from salam.module_MD.D_Pi_A_Enumeration import delete_repeated_smiles_of_mols

from salam.module_MD.OrganoMetalics_Enumeration import mutations_CH_aromatic
from salam.module_MD.OrganoMetalics_Enumeration import mutations_CH_aromatic_termino_H
import subprocess

import numpy as np
#from scipy.special import comb
#import os

import argparse

parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--carbon_sub', type=str, default='N' )
parser.add_argument('--termino_hydrogen_subs', type=list, default=[ '*F',  '*C#N',  '*OC',  '*N(C)C' ] )

parser.add_argument('--GXXXX', type=str, default="G0000")
parser.add_argument('--PXXXX', type=str, default="P0000")
parser.add_argument('--OXXXX', type=str, default="O0000")
parser.add_argument('--GXXXXPLUSONE', type=str, default="G0001")

args = parser.parse_args()

print("args.carbon_sub: ", args.carbon_sub, type(args.carbon_sub))
CARBON_SUB = args.carbon_sub

print("args.termino_hydrogen_subs: ", args.termino_hydrogen_subs, type(args.termino_hydrogen_subs))
TERMINO_HYDROGEN_SUBS = args.termino_hydrogen_subs

#TERMINO_HYDROGEN_SUBS  = [ ele.strip()  for ele in TERMINO_HYDROGEN_SUBS.strip().split(",") ]


print(args.GXXXX, type(args.GXXXX))
print(args.PXXXX, type(args.PXXXX))
print(args.OXXXX, type(args.OXXXX))
GXXXX = args.GXXXX
PXXXX = args.PXXXX
OXXXX = args.OXXXX

print(args.GXXXXPLUSONE, type(args.GXXXXPLUSONE))
GXXXXPLUSONE = args.GXXXXPLUSONE


# script_call_path = os.getcwd()
# print("script_call_path:\n", script_call_path)
# print("script name:\n")
# print(__file__)
# print("change to ../")
# os.chdir( "../" )



def random_sampling_mols(All_mols, sample_number=1000):
    #------random sampling 1000 mols------------------------------------------
    print("The length of All_mols is : ", len(All_mols))
    if (sample_number >= 1) & ( len(All_mols) >= sample_number):
        sample_indexs = np.random.choice(len(All_mols), sample_number, replace=False)
        All_mols_sample = [All_mols[idx] for idx in sample_indexs]
        All_mols = All_mols_sample 
        print("After sampling, the length of All_mols is : ", len(All_mols))
    else:
        print("The length of All_mols is  smaller than sample_number: ", len(All_mols) , "   <   ",  sample_number )
    
    return All_mols


#------ main function -----------------------------------------------------
#--------------------------------------------------------------------------

#----- reading opt_mols sdf file -----------------
opt_sdf_path = './project/'  +  GXXXX  +   '/'  +  OXXXX   + '.sdf'
sdsuppl = Chem.SDMolSupplier(opt_sdf_path)
opt_mols = [x for x in sdsuppl if x is not None]
print("length of opt_mols is : ", len(opt_mols))


#-----------------------------------------------------------------------------
#---- create combinational compound library --------------------------
#---- via calling mutations_CH_aromatic() or mutations_CH_aromatic_termino_H()

print("\n", '*'*80)

All_mols = []


carbon_or_hydrogen_subs = [ CARBON_SUB ] +  TERMINO_HYDROGEN_SUBS

subs_mols = [ [] for i in range(len(carbon_or_hydrogen_subs))]



for opt_mol  in opt_mols[:]:
    mutation_mols = mutations_CH_aromatic(opt_mol, 
                                          patt=Chem.MolFromSmarts('c[H]'), 
                                          substitute_atom=CARBON_SUB,
                                          substitute_atomnum=7, 
                                          substi_number=1, 
                                          sample_number=3, 
                                          #is_showmol=True, 
                                          max_subpos=10
                                          )
    
    All_mols +=  [ opt_mol ] +  mutation_mols
    subs_mols[0] +=  mutation_mols
    
    for termino_hydrogen_sub, i in zip( TERMINO_HYDROGEN_SUBS,  range(1, len(carbon_or_hydrogen_subs)) ):
        mutation_mols1 = mutations_CH_aromatic_termino_H(opt_mol, 
                                                        patt=Chem.MolFromSmarts('c[H]'), 
                                                        substitute_group=Chem.MolFromSmiles( termino_hydrogen_sub ), 
                                                        substi_number=1, 
                                                        sample_number=3, 
                                                        #is_showmol=True,
                                                        max_subpos=10
                                                        )
        
        subs_mols[i]  +=  mutation_mols1
        
        All_mols +=  mutation_mols1
    

print("The length of All_mols is : ", len(All_mols))


draw_mols(All_mols, 
          16, 
          './project/%s/images/foo011--mutation-mols-demo.png'%GXXXX, 
          molsPerRow=4, 
          size=(800, 800)
          )


All_mols  =  delete_repeated_smiles_of_mols(All_mols)
print("After delete_repeated_smiles_of_mols, the length of All_mols is : ", len(All_mols))

new_subs_mols = []
for i in range(len(carbon_or_hydrogen_subs)):
    subs_mols[i]  =  delete_repeated_smiles_of_mols(subs_mols[i])
    
    #subs_mols[i] = [ mol for mol in opt_mols ]
    new_sub_mols = list( set(subs_mols[i]).difference(set(opt_mols)) )
    new_subs_mols.append( new_sub_mols )



#------random sampling 1000 mols-----------------------------------------------------
All_mols = random_sampling_mols(All_mols, sample_number=1000)

cpd_names = [ "cpd-%05d" % (i+1) for i in range(len(All_mols)) ]
for mol, cpd_name in zip(All_mols, cpd_names):
    mol.SetProp("_Name", cpd_name )


#---- write molecules to SD File ------------------------------- 
call_mkdir_GXXXXplus1 = 'mkdir  ./project/' +  GXXXXPLUSONE  +  '/  2>/dev/null  '
status, output = subprocess.getstatusoutput( call_mkdir_GXXXXplus1 )
print("mkdir  ./project/GXXXX+1/;    status, output = ", status, output)

outfile_path = './project/'   +   GXXXXPLUSONE   +  '/'   + GXXXXPLUSONE + '.sdf'
All_mols_w = Chem.SDWriter(outfile_path)
for m in tqdm(All_mols): 
    m_temp = Chem.AddHs(m)
    m1_temp, nb_try_EmbedMolecule = try_embedmolecule(m_temp)
    All_mols_w.write(m1_temp)




print("\nEnd of program: PG011!\n")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Jul  2 15:26:07 2023

@author: tucy
"""


from rdkit import Chem

from salam.module_MD.D_Pi_A_Enumeration import delete_repeated_smiles_of_mols
# from D_Pi_A_Enumeration import try_embedmolecule
from salam.module_MD.D_Pi_A_Enumeration import write_mols_paralell
# from Crossover import crossover_two_mols_break_singlebond
# from Crossover import crossover_two_mols_break_twobondsring
from salam.module_MD.Crossover import crossover_mols_parallel
from salam.module_MD.Crossover import screen_unfavored_rings
from salam.module_MD.Crossover import get_numatoms

from time import time
import argparse


parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--output_lib_type', type=str, default="csv")

args = parser.parse_args()

print("output_lib_type: ", args.output_lib_type, type(args.output_lib_type))
output_lib_type = args.output_lib_type

#------------output_lib_type = "sdf" or "csv"-------------
#------------ "csv" is used for test purpose 
#------------ "sdf" is used for practical generation of compound library, which can be used for subsequent computation.  
#------------ output_lib_type = "csv"



#_________________________________________________________________________________
#*********************************************************************************
#-----__main__ function begins here ----------------------------------------------
if __name__ == '__main__':

    print("Begins")
    print("-"*80)

    start_time = time()


    Slibs, Dlibs, SDlibs = crossover_mols_parallel(parent_mols_smis='./parent-mols.smi',
                                                   MolWt_expansion_ratios=[-0.20, 0.20],
                                                   RS_LIMITS=[4, 7], 
                                                   NAR_SHIFTS=[-1, 1],
                                                   type1_smarts=['[a&R]-[R]', '[a&R]-[!R]'],    
                                                   type2_smarts=['a1:a:a:a:a:a1', 'a1:a:a:a:a1'],
                                                   )


    #--- screen_unfavored_rings -------------------------------------------
    SDlibs_rev, SDlibs_delete = screen_unfavored_rings(SDlibs, 
                                                       smarts_list=['[#16&r6]', '[#34&r6]'],
                                                       )
    

    all_libs = [ Slibs, Dlibs, SDlibs, SDlibs_rev, SDlibs_delete ]
    sdf_names = ['./Slibs.sdf', './Dlibs.sdf', './SDlibs.sdf', './SDlibs_rev.sdf', './SDlibs_delete.sdf']
    csv_names = ['./Slibs.csv', './Dlibs.csv', './SDlibs.csv', './SDlibs_rev.csv', './SDlibs_delete.csv']
    dir_names = ['./Slibs', './Dlibs', './SDlibs', './SDlibs_rev', './SDlibs_delete']

    
    print('^'*60)
    print("### Summary of libs:")    
    for tmplibs, sdf_name, csv_name, dir_name  in zip(all_libs, sdf_names, csv_names, dir_names):
        print("len(tmplibs) = ", len(tmplibs) )
        tmplibs = delete_repeated_smiles_of_mols(tmplibs, is_sanitize=True)
           
        print("After deleting redundance, len(tmplibs) = ", len(tmplibs) )
           
        print("Sort list by NumAtoms.")
        tmplibs = sorted(tmplibs, 
                         key=get_numatoms, 
                         reverse=False
                         )
           
        print("len(tmplibs) = ", len(tmplibs))
           
        print('-'*30)
        print("!!! The complete set of reconstructed molecules in tmplibs.")
        for mol,i in zip(tmplibs,range(len(tmplibs))):
            print(i+1, " : ",  Chem.MolToSmiles(mol))
    
    
        #---- set Names for mols ------------------------------- 
        All_mols1 = tmplibs
        All_smis1 = [ Chem.MolToSmiles(mol) for mol in All_mols1 ]
        cpd_names1 = [ "cpd-%05d" % (i+1) for i in range(len(All_mols1)) ]
        for mol, cpd_name in zip(All_mols1, cpd_names1):
            mol.SetProp("_Name", cpd_name )
        
        if (output_lib_type == "sdf") or (output_lib_type == "SDF"): 
            
            write_mols_paralell(All_mols1, 
                                cpd_names1, 
                                outfile_path='%s/'%dir_name
                                )
            
        
        elif (output_lib_type == "csv") or (output_lib_type == "CSV"): 
            with open(csv_name, 'w') as fp:
                fp.write("cpdname,smile\n")
                for cpd_name, smi in zip(cpd_names1, All_smis1):
                    fp.write("%s,%s\n"%(cpd_name,smi))
        
        else:
            print("Parameter output_lib_type must be either csv or sdf.")
            


    end_time = time()
    run_time = (end_time - start_time)/60.
    print("### The CPU time is %15.3f min.\n"%run_time)

    print("\nFinished!")      



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Jul  2 15:26:07 2023

@author: tucy
"""


from rdkit import Chem

from salam.module_MD.D_Pi_A_Enumeration import delete_repeated_smiles_of_mols
from salam.module_MD.D_Pi_A_Enumeration import write_mols_paralell
from salam.module_MD.OrganoMetalics_Enumeration import mutual_mutations_mols
from salam.module_MD.Crossover import get_numatoms


from time import time
import argparse


parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--output_lib_type', type=str, default="csv")

args = parser.parse_args()

print("output_lib_type: ", args.output_lib_type, type(args.output_lib_type))
output_lib_type = args.output_lib_type

#------------output_lib_type = "sdf" or "csv"-------------
#------------ "csv" is used for test purpose 
#------------ "sdf" is used for practical generation of compound library, which can be used for subsequent computation.  
#------------ output_lib_type = "csv"



#_________________________________________________________________________________
#*********************************************************************************
#-----__main__ function begins here ----------------------------------------------
if __name__ == '__main__':

    print("Begins")
    print("-"*80)

    start_time = time()

    basemol_suppl = Chem.SmilesMolSupplier('./parent-mols.smi',
                                           titleLine=False
                                           )
    
    base_mols = [x for x in basemol_suppl if x is not None]
    
    print("len(base_mols) = ", len(base_mols))


    Mut_libs = mutual_mutations_mols(BASE_MOLS=base_mols, 
                                     CARBON_SUB='N', 
                                     TERMINO_HYDROGEN_SUB='*C#N',
                                     MAX_SUBPOS=16, 
                                     SUBSTI_NUMBER_CARBON=0, 
                                     SAMPLE_NUMBER_CARBON=0, 
                                     SUBSTI_NUMBER_HYDROGEN=0, 
                                     SAMPLE_NUMBER_HYDROGEN=0)


    sdf_name = './Mut_libs.sdf'
    csv_name = './Mut_libs.csv'
    dir_name = './Mut_libs'

    
    print('^'*60)
    print("### Summary of libs:")    
    print("len(Mut_libs) = ", len(Mut_libs) )
    Mut_libs = delete_repeated_smiles_of_mols(Mut_libs, is_sanitize=True)
       
    print("After deleting redundance, len(Mut_libs) = ", len(Mut_libs) )
       
    print("Sort list by NumAtoms.")
    Mut_libs = sorted(Mut_libs, 
                     key=get_numatoms, 
                     reverse=False
                     )
       
    print("len(Mut_libs) = ", len(Mut_libs))
       
    print('-'*30)
    print("!!! The complete set of reconstructed molecules in Mut_libs.")
    for mol,i in zip(Mut_libs,range(len(Mut_libs))):
        print(i+1, " : ",  Chem.MolToSmiles(mol))


    #---- set Names for mols ------------------------------- 
    All_mols1 = Mut_libs
    All_smis1 = [ Chem.MolToSmiles(mol) for mol in All_mols1 ]
    cpd_names1 = [ "cpd-%05d" % (i+1) for i in range(len(All_mols1)) ]
    for mol, cpd_name in zip(All_mols1, cpd_names1):
        mol.SetProp("_Name", cpd_name )
    
    if (output_lib_type == "sdf") or (output_lib_type == "SDF"): 
        
        write_mols_paralell(All_mols1, 
                            cpd_names1, 
                            outfile_path='%s/'%dir_name
                            )
        
    
    elif (output_lib_type == "csv") or (output_lib_type == "CSV"): 
        with open(csv_name, 'w') as fp:
            fp.write("cpdname,smile\n")
            for cpd_name, smi in zip(cpd_names1, All_smis1):
                fp.write("%s,%s\n"%(cpd_name,smi))
    
    else:
        print("Parameter output_lib_type must be either csv or sdf.")
            


    end_time = time()
    run_time = (end_time - start_time)/60.
    print("### The CPU time is %15.3f min.\n"%run_time)

    print("\nFinished!")      



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Apr 16 10:15:03 2021

@author: tucy
"""

import re
import glob
#import numpy as np
import pandas as pd

#from rdkit import Chem
#from rdkit.Chem import AllChem
#import subprocess
#import shutil
import os
import argparse
#import time
import subprocess

parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--analyzed_property', type=str, default="tadf")
parser.add_argument('--pbs_job_incompleteness_tolerance', type=float, default=0.01)
parser.add_argument('--sleep_seconds', type=int, default=300)
parser.add_argument('--GXXXX', type=str, default="G0000")
parser.add_argument('--PXXXX', type=str, default="P0000")
parser.add_argument('--OXXXX', type=str, default="O0000")

args = parser.parse_args()

print("analyzed_property: ", args.analyzed_property, type(args.analyzed_property) )
print("pbs_job_incompleteness_tolerance: ", args.pbs_job_incompleteness_tolerance, type(args.pbs_job_incompleteness_tolerance) )

analyzed_property = args.analyzed_property
pbs_job_incompleteness_tolerance = args.pbs_job_incompleteness_tolerance

print(args.GXXXX, type(args.GXXXX))
print(args.PXXXX, type(args.PXXXX))
print(args.OXXXX, type(args.OXXXX))
GXXXX = args.GXXXX
PXXXX = args.PXXXX
OXXXX = args.OXXXX

print(args.sleep_seconds, type(args.sleep_seconds))
SLEEP_SECONDS = args.sleep_seconds


 
    
#--------------------------------------------------------------------------------------------------
def get_normal_lognames(pbs_jobs_wkdir='./project/%s/pick_mols/com-files/finished/com-files/'%GXXXX, 
                        project_dir='../../../../../../'):
    
    os.chdir( pbs_jobs_wkdir ) 
    
    filenames = glob.glob(r'./cpd*.log')
    
    if (len(filenames) == 0):
        print("No cpd*.log file exit! The ruturn is set to 0.")
        exit(1)
    else:
        failed_log_idxs = []
        
        idx = 0
        for filename  in  filenames:
            termination_pattern = re.compile(r'Normal(\s*)termination(.*?)', re.MULTILINE|re.DOTALL)
            with open(filename) as fp:
                fc=fp.read()
                termination_result = termination_pattern.findall(fc)
                #print("# ", filename, "", len(termination_result))
                if len(termination_result) != 1:
                    failed_log_idxs.append(idx)
                
                idx += 1
        
        os.chdir( project_dir ) 
        
        #normal_logfile_names = [x.replace(r'./', '').replace(r'.log', '')   for  x in filenames]
        normal_logfile_names = [x  for x in filenames if not None]
        
        if len(failed_log_idxs) > 0:
            print('-'*60)
            print("# failed_compound_names: ")
            for failed_log_idx in failed_log_idxs:            
                print(filenames[failed_log_idx])
        
            
            failed_log_idxs.sort(reverse=True)
            print("# pop failed log names:")
            for i  in failed_log_idxs:
                normal_logfile_names.pop(i)
                print("idx = ", i)
            
            print("len(normal_logfile_names) = ", len(normal_logfile_names))
            return normal_logfile_names
        else:
            print("len(normal_logfile_names) = ", len(normal_logfile_names))
            return normal_logfile_names



def analysis_property_tadf(normal_logfile_names, 
                           pbs_jobs_wkdir='./project/%s/pick_mols/com-files/finished/com-files/'%GXXXX, 
                           project_dir='../../../../../../'):
    
    print("Hello in func: analysis_property_tadf")
    
    os.chdir( pbs_jobs_wkdir ) 
    
    filenames = normal_logfile_names

    tadf_results = []
    
    for filename  in  filenames:
        tadf_pattern = re.compile(r'Excitation(\s*)energies(\s*)and(\s*)oscillator(\s*)strengths(.*?)Population(\s*)analysis(\s*)using', re.MULTILINE|re.DOTALL)
        with open(filename) as fp:
            fc=fp.read()
            tadf_result = tadf_pattern.findall(fc)
            print("# ", filename, "", len(tadf_result))
            tadf_results.append(tadf_result)           
      
    #------------------- extract data from strings, and store data in pd.DataFrame 
    
    all_T1_energies = []
    all_S1_energies = []
    all_S1_frequencies = []
    
    all_TEn_SEn_SFrs = []
    
    for tadf_result  in  tadf_results:
        tadf_result = str(tadf_result)
        
        # print('-'*60)
        # print("The original string is: \n",  tadf_result )
        
        tadf_result = tadf_result.split(r'\n')
        # print('-'*60)
        # print("The splitted string is: \n",  tadf_result )
        
        excited_state_pattern = re.compile(r'^(\s*)Excited(\s*)State(.*?)<S\*\*2>=(\S*?)', re.DOTALL) 
        tadf_result_new = []
        for ele in tadf_result:
            ele_new = excited_state_pattern.findall(ele)
            if len(ele_new) > 0:
                tadf_result_new.append(ele_new[0])
        
        # print('-'*60)
        # print("The transformed tadf_result_new string is: \n", tadf_result_new)
        
        
        # if (len(tadf_result_new) > 0):
        #     print('-'*60)
        #     for  i  in range(len(tadf_result_new)):
        #         print(i, "   ",  tadf_result_new[i])
        
        
        tadf_result_new1 = [ele[2].strip() for ele in   tadf_result_new]
        # print('-'*60)
        # print("The transformed tadf_result_new1 is:")
        # for ele in tadf_result_new1:
        #     print(ele)
        
        # print("len is : ", len(tadf_result_new1) )
        
        triplet_state_pattern = re.compile(r'(\s*)Triplet-\w(.*?)$', re.DOTALL) 
        singlet_state_pattern = re.compile(r'(\s*)Singlet-\w(.*?)$', re.DOTALL) 
        
        triplet_states = []
        for ele in tadf_result_new1:
            ele_new = triplet_state_pattern.findall(ele)
            if len(ele_new) > 0:
                triplet_states.append(ele_new[0])
        
        singlet_states = []
        for ele in tadf_result_new1:
            ele_new = singlet_state_pattern.findall(ele)
            if len(ele_new) > 0:
                singlet_states.append(ele_new[0])
          
        
        # print('-'*60)
        # print("The transformed triplet_states is:")
        # for ele in triplet_states:
        #     print(ele)
            
        # print('-'*60)
        # print("The transformed singlet_states is:")
        # for ele in singlet_states:
        #     print(ele)
        
        T1_energies = [ele[-1].strip().split()[0] for ele in triplet_states]
        S1_energies = [ele[-1].strip().split()[0] for ele in singlet_states]
        S1_frequencies = [ele[-1].strip().split()[4].split("=")[1] for ele in singlet_states]
        
        TEn_SEn_SFrs = []
        for ele1, ele2, ele3 in zip(T1_energies, S1_energies, S1_frequencies):
            #print(ele1, ele2, ele3)
            TEn_SEn_SFrs.append(ele1)
            TEn_SEn_SFrs.append(ele2)
            TEn_SEn_SFrs.append(ele3)
        
           
        print('-'*60)
        print(len(T1_energies), len(S1_energies), len(S1_frequencies))
    
        
        #print('-'*60)
        print("len(TEn_SEn_SFrs) is ", len(TEn_SEn_SFrs))
        
        
        all_TEn_SEn_SFrs.append(TEn_SEn_SFrs)
        all_T1_energies.append(T1_energies)
        all_S1_energies.append(S1_energies)
        all_S1_frequencies.append(S1_frequencies)
        
    
    
    compoundnames = [x.replace(r'./', '').replace(r'.log', '')   for  x in filenames]
    
    # for name in compoundnames:
    #     print(name)
    
    datanames = []
    for i in range(len(all_TEn_SEn_SFrs[0][:])//3) :
        datanames.append('T%d_en'%(i+1) )
        datanames.append('S%d_en'%(i+1)  )
        datanames.append('S%d_freq'%(i+1) )
           
       
    df = pd.DataFrame(data=all_TEn_SEn_SFrs, columns=datanames)
    df["compound"] = compoundnames
    df.set_index("compound", inplace=True)
    
    print(df.shape)
    print(df)

    os.chdir( project_dir ) 
    
    return df


def analysis_property_nlo(normal_logfile_names, 
                          pbs_jobs_wkdir='./project/%s/pick_mols/com-files/finished/com-files/'%GXXXX, 
                          project_dir='../../../../../../'):
    
    print("Hello in func: analysis_property_nlo")
    
    os.chdir( pbs_jobs_wkdir ) 
    
    filenames = normal_logfile_names

    nlo_results = []
    
    for filename  in  filenames:
        nlo_pattern = re.compile(r'\\V(.*?)\\@$', re.MULTILINE|re.DOTALL)
        with open(filename) as fp:
            fc=fp.read()
            nlo_result = nlo_pattern.findall(fc)
            print("# ", filename, "", len(nlo_result))
            nlo_results.append(nlo_result)

            
    #------------------- extract data from strings, and store data in pd.DataFrame 
    HFs = []
    polars = []
    hyperpolars = []
    HF_polar_hyperpolars = []
    
    for nlo_result  in  nlo_results:
        nlo_result = str(nlo_result)
        
        #print("The original sting is: \n",  nlo_result )
        
        nlo_result = nlo_result.replace('\n', '').replace('\r', '').replace(' ', '').split('=')    
        #print('-'*60)
        #print("The transformed sting is: \n", nlo_result)
        
        print("len is : ", len( nlo_result ))
        
        HF = nlo_result[3].strip().replace(r'\n', '').replace('\r', '').replace(' ', '').split(r'\\')[0].replace(' ', '')
        polar = nlo_result[6].strip().replace(r'\n', '').replace('\r', '').replace(' ', '').split(r'\\')[0].replace(' ', '')
        hyperpolar = nlo_result[7].strip().replace(r'\n', '').replace('\r', '').replace(' ', '').split(r'\\')[0].replace(' ', '')
            
        
        HF = HF.split(',')
        polar = polar.split(',')
        hyperpolar = hyperpolar.split(',')
        
        print('-'*60)
        print(len(HF), len(polar), len(hyperpolar))
    
        HFs.append(HF)
        polars.append(polar)
        hyperpolars.append(hyperpolar)
        
        HF_polar_hyperpolar = HF + polar + hyperpolar
        HF_polar_hyperpolars.append(HF_polar_hyperpolar)
    
    
    compoundnames = [x.replace(r'./', '').replace(r'.log', '')   for  x in filenames]
    
    # for name in compoundnames:
    #     print(name)
    
    datanames = ["hf", "axx", "axy", "ayy", "axz", "ayz", "azz", \
        "bxxx", "bxxy", "bxyy", "byyy", "bxxz", "bxyz", "byyz", "bxzz", "byzz", "bzzz"]
        
        
    df = pd.DataFrame(data=HF_polar_hyperpolars, columns=datanames)
    df["compound"] = compoundnames
    df.set_index("compound", inplace=True)
    
    print(df.shape)
    print(df)
    
    os.chdir( project_dir ) 

    return df
#----------------------------------------------------------------------------------------




#--main func -----------------------------------------------------------------------------

base_wkdir = os.getcwd()
print("base_wkdir: \n", base_wkdir)


#-------main loop ------------------------------------------------------------
print('*'*60)
print("# Begin PG009__property_analysis_summary.")

GXXXX_pick_mols_com_files_finished_com_files_dir =  './project/'  +   GXXXX  +  '/pick_mols/com-files/finished/com-files/'


normal_logfile_names = get_normal_lognames(pbs_jobs_wkdir=GXXXX_pick_mols_com_files_finished_com_files_dir, 
                                           project_dir=base_wkdir,)


print("\nNormal_logfile_names: ")
for inf in normal_logfile_names:
    print(inf)


#-----------------------------------------------------------------------------

os.chdir(base_wkdir)
CWD = os.getcwd()
print("\nCWD: \n", CWD)
#------------------------------------------------------------------------------


print("# Begin analyzing property.\n")

if (analyzed_property == "tadf") or (analyzed_property == "TADF"):
    
    df_tadf = analysis_property_tadf(normal_logfile_names, 
                                     pbs_jobs_wkdir='./project/%s/pick_mols/com-files/finished/com-files/'%GXXXX, 
                                     project_dir=base_wkdir
                                     )
    
    print('-'*60)
    tadf_summary_file_path = './project/' + GXXXX +  '/tadf_summary_'   +   PXXXX  +  '.csv'
    print("The summary of property analysis TADF is stored at: ")
    print(tadf_summary_file_path)
    
    with open(tadf_summary_file_path, 'w') as tmp:
        df_tadf.to_csv(tmp)

elif (analyzed_property == "nlo") or (analyzed_property == "NLO"):
    
    df_nlo = analysis_property_nlo(normal_logfile_names, 
                                    pbs_jobs_wkdir='./project/%s/pick_mols/com-files/finished/com-files/'%GXXXX, 
                                    project_dir=base_wkdir
                                    )
    
    print('-'*60)
    nlo_summary_file_path = './project/' + GXXXX +  '/nlo_summary_'   +   PXXXX  +  '.csv'
    print("The summary of property analysis NLO is stored at: ")
    print(nlo_summary_file_path)
    
    with open(nlo_summary_file_path, 'w') as tmp:
        df_nlo.to_csv(tmp)

else:
    print("No defined parameter.")    
    print("analyzed_property: tadf or nlo.")
    exit(1)
    

print('-'*60)
src_file_path = './project/'    +   GXXXX  +  '/pick_mols/sdf-files/'
GXXXX_PXXXXrev_sdf_path =  './project/'   +  GXXXX  +  '/'  +  PXXXX  + 'rev.sdf'

print("\nThe revised pick_mols is stored at %s"%GXXXX_PXXXXrev_sdf_path)

call_rm_GXXXX_PXXXXrev_sdf_path  =  'rm  -f  '  +  GXXXX_PXXXXrev_sdf_path  + '   2>/dev/null  '
status, output = subprocess.getstatusoutput( call_rm_GXXXX_PXXXXrev_sdf_path )
print("rm -f   %s.    status, output = "%GXXXX_PXXXXrev_sdf_path, status, output)


normal_compoundnames = [x.replace(r'./', '').replace(r'.log', '')   for  x in  normal_logfile_names]

for normal_name in normal_compoundnames:
    #print(normal_name)
    normal_sdf_file_name = src_file_path + normal_name + ".sdf"
    cat_file_to_Pickmols = 'cat  '   +  normal_sdf_file_name   +   '   >>  ./project/'  +  GXXXX  + '/'   +  PXXXX   +  'rev.sdf   '
    status, output = subprocess.getstatusoutput( cat_file_to_Pickmols )
    #print("status, output = ", status, output)
    

print("\n### End of program:  PG009!\n")




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Apr 23 21:50:49 2021

@author: tucy
"""

import sys 
import os
#os.chdir('../')
script_wkdir = os.getcwd()
sys.path.append(script_wkdir)


import argparse

parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--prop_tendency', type=str, default="small")
parser.add_argument('--GXXXX', type=str, default="G0000")
parser.add_argument('--PXXXX', type=str, default="P0000")
parser.add_argument('--OXXXX', type=str, default="O0000")

args = parser.parse_args()
print(args.GXXXX, type(args.GXXXX))
print(args.PXXXX, type(args.PXXXX))
print(args.OXXXX, type(args.OXXXX))
GXXXX = args.GXXXX
PXXXX = args.PXXXX
OXXXX = args.OXXXX

print(args.prop_tendency, type(args.prop_tendency))
prop_tendency = args.prop_tendency


#import os
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
#import tensorflow as tf

import deepchem as dc
import numpy as np
import pandas as pd
#import tempfile
#from sklearn.ensemble import RandomForestRegressor
from rdkit import Chem
#from rdkit.Chem import Draw, AllChem
#from rdkit.Chem import rdDepictor
from salam.module_MD.D_Pi_A_Enumeration import draw_mols
import joblib
from sklearn.impute import SimpleImputer
import matplotlib.pyplot as plt
import subprocess


#----------------------------------------------------------------------------------
def load_sdf_to_df(sdf_path='./data/All_mols.sdf'):
    #-------- loading  sdf file ---------------------------------------------------
    sdsuppl = Chem.SDMolSupplier(sdf_path)
    #sdsuppl = Chem.SDMolSupplier('./data/Prod_mols.sdf')
    
    mols = [x for x in sdsuppl if x is not None]
    smiles = [Chem.MolToSmiles(mol) for mol in mols]
    compoundnames = [mol.GetProp("_Name") for mol in mols]
    properties = [np.nan for i in range(len(mols))]
    print("-"*60)  
    print( "len(mols) is ", len(mols) ) 
    print("len(properties) is ", len(properties))
    
    df_mols = pd.DataFrame(data=zip(mols, smiles, compoundnames), columns=["mols", "smiles", "compoundnames"])
    
    df_mols.set_index("compoundnames", inplace=True)
    
    print(df_mols.head())
    
    return mols, smiles, compoundnames, properties, df_mols


def impute_dataset(X, strategy="median"):
    imputer = SimpleImputer(strategy=strategy)
    imputer.fit(X)
    X_imputed = imputer.transform(X)
    print("type(X_imputed) is ", type(X_imputed))
    
    return X_imputed 


def creat_ECFP_RDKITDES_CM_features_dataset(mols, smiles, properties, is_compute_CM=True):   
    #--------- ECFP ---------------------------------------------------
    featurizer = dc.feat.CircularFingerprint(size=2048)
    ecfp = featurizer.featurize(smiles)
    dataset = dc.data.NumpyDataset(X=ecfp, y=np.array(properties))
    print("ecfp.shape is ", ecfp.shape )
    print("len(dataset) is ", len(dataset) ) 
    
    #-------------------------------------------------------------------
    #------- RDKitDescriptors ------------------------------------------
    rdkit_featurizer = dc.feat.RDKitDescriptors()
    rdkit_des_features = rdkit_featurizer(smiles)
    dataset1 = dc.data.NumpyDataset(X=rdkit_des_features, y=np.array(properties))
    print("len(dataset1) is ", len(dataset1) ) 
    
    print("-"*60) 
    print("RDKit Descriptors and values for a compound.") 
    for feature, descriptor in zip(rdkit_des_features[0][:], rdkit_featurizer.descriptors):
        print(descriptor, feature)
    
    print('The number of sample present is: ', len(rdkit_des_features))
    print('The number of descriptors present is: ', len(rdkit_des_features[0]))
    
    print("rdkit_des_features.shape is ", rdkit_des_features.shape)
    
    #------------------------------------------------------------------------------
    #------- CoulombMatrix --------------------------------------------------------
    if (is_compute_CM == True):
        coulomb_mat = dc.feat.CoulombMatrix(max_atoms=100)
        coulomb_mat_features = coulomb_mat(mols)
        
        print("type(coulomb_mat_features) ", type(coulomb_mat_features))
        print("coulomb_mat_features.shape is ", coulomb_mat_features.shape)
        
        coulomb_mat_features = coulomb_mat_features.reshape(coulomb_mat_features.shape[0],
                                                            coulomb_mat_features.shape[1]*coulomb_mat_features.shape[2])
        
        print("coulomb_mat_features.shape is ", coulomb_mat_features.shape)
        
        dataset2 = dc.data.NumpyDataset(X=coulomb_mat_features, y=np.array(properties))
        print("len(dataset2) is ", len(dataset2) ) 
        
        return  dataset, dataset1, dataset2
    else:
        return  dataset, dataset1


#------ load and predict ------------------------------------------
def load_and_predict(df, dataset, model_path=None, ascending=False):

    if model_path is not None:
        model_loaded = joblib.load(model_path)
    else:
        print("loading model %s failed!" % model_path)
        exit(-1)

    # model evaluate on  all molecules. 
    prop_preds = model_loaded.predict(dataset.X)
    print( "len(prop_preds) is ", len(prop_preds) )
    
    df["prop_preds"] = prop_preds
    
    df.sort_values(by="prop_preds", ascending=ascending, inplace=True)

    print('# The optimal 16 molecules are: ')
    print( df.head(16) )
    
    return df


def draw_optimal_mols(df, imgname='./images/foo009-optimal-mols.png', unit=' a.u. '):
    opt_mols = df["mols"].tolist()[:16]
    props_values = df["prop_preds"].tolist()[:16] 
    props_as_legends = [" " + str(round(number, 3)) + unit for number in props_values]
    
    draw_mols(opt_mols, 
              16, 
              imgname, 
              molsPerRow=4, 
              size=(800, 800),
              legends=props_as_legends,
              )


def plot_prop_pred(df, figname='./images/foo009-property-statistics.png', xlabel_name='Energy gap'):
    ax1 = df['prop_preds'].plot(title='Distribution of property',
                                  kind='hist',
                                )
    
    plt.xlabel(xlabel_name)
    #plt.ylabel('Frequency')
    
    fig1 = ax1.get_figure() 
    fig1.savefig(figname)
    #plt.show()
    #plt.pause(10)
    plt.close()



#---main function begins --------------------------------------------------------
#--------------------------------------------------------------------------------

print("-"*60) 
GXXXX_sdf_path = './project/'  +  GXXXX    +  '/'  + GXXXX  +  '.sdf'
print("Loading sdf for ML prediction, the path is:\n", GXXXX_sdf_path)

mols, smiles, compoundnames, properties, df_mols = load_sdf_to_df(sdf_path=GXXXX_sdf_path )

dataset, dataset1 = creat_ECFP_RDKITDES_CM_features_dataset(mols, 
                                                            smiles, 
                                                            properties, 
                                                            is_compute_CM=False
                                                            )

print("The ECFP model is loaded for prediction of whole mols.")

X_imputed = impute_dataset(dataset.X, strategy="median")
dataset_imputed = dc.data.NumpyDataset(X=X_imputed, y=np.array(properties)) 

if (prop_tendency == "small"):
    df_ECFP = load_and_predict(df_mols, 
                               dataset_imputed, 
                               model_path='./project/%s/models/ECFP/rf_reg_model_ECFP.pkl'%GXXXX, 
                               ascending=True
                               )
elif (prop_tendency == "large"):
    df_ECFP = load_and_predict(df_mols, 
                               dataset_imputed, 
                               model_path='./project/%s/models/ECFP/rf_reg_model_ECFP.pkl'%GXXXX, 
                               ascending=False
                               )
else:
    print("prop_tendency: small or large.")    
    exit(1)


print("The optimal mols is shown in fig: \n", './project/%s/images/foo-ECFP-optimal-mols-in-all-mols.png'%GXXXX )
draw_optimal_mols(df_ECFP, 
                  imgname='./project/%s/images/foo-ECFP-optimal-mols-in-all-mols.png'%GXXXX, 
                  unit=' eV '
                  )

print("The property statistics is shown in fig: \n", './project/%s/images/foo-property-statistics-in-all-mols.png'%GXXXX )
plot_prop_pred(df_ECFP, 
               figname='./project/%s/images/foo-property-statistics-in-all-mols.png'%GXXXX, 
               xlabel_name='Energy gap'
               )


#---- write df_ECFP to csv-----------------------------------------------------
print('-'*60)
print(df_ECFP.head() )
GXXXX_df_ECFP_csv_path = './project/'  +  GXXXX    +  '/'  + GXXXX   +  '_df_ECFP.csv'
call_rm_GXXXX_df_ECFP_csv_file =  'rm -f  '  +  GXXXX_df_ECFP_csv_path  +  '   2>/dev/null  '
status, output = subprocess.getstatusoutput( call_rm_GXXXX_df_ECFP_csv_file )
print("rm -f  %s.    status, output = "%GXXXX_df_ECFP_csv_path,  status, output)
print("df_ECFP is stored at %s\n"%GXXXX_df_ECFP_csv_path )

with open(GXXXX_df_ECFP_csv_path, 'w') as tmp:
    df_ECFP.to_csv(tmp)
#-----------------------------------------------------------------------------


optimal_cpdnames = df_ECFP.index.tolist()[0:100]
print("-"*60)
print("The optimal_cpdname are:")
for cpdname in optimal_cpdnames:
    print(cpdname)



print('-'*60)
src_file_path = './project/%s/separate_mols/'%GXXXX
print("\nThe optimal_cpds is stored at ./project/%s/%s.sdf"%(GXXXX, OXXXX) )

call_rm_GXXXX_OXXXX_sdf_file =  'rm -f  ./project/'  +  GXXXX  +  '/'   +  OXXXX  +  '.sdf 2>/dev/null  '
status, output = subprocess.getstatusoutput( call_rm_GXXXX_OXXXX_sdf_file )
print("rm -f  ./project/%s/%s.sdf.    status, output = "%(GXXXX, OXXXX), status, output)

for normal_name in optimal_cpdnames:
    #print(normal_name)
    normal_sdf_file_name = src_file_path + normal_name + ".sdf"
    cat_file_to_Pickmols = "cat  "  +  normal_sdf_file_name   +  "   >>  "  +  "./project/"  +  GXXXX  + "/"  +  OXXXX  +  ".sdf"
    status, output = subprocess.getstatusoutput( cat_file_to_Pickmols )
    #print("status, output = ", status, output)



print("\n### End of program: PG011!\n")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov  8 09:20:07 2021

@author: tucy
"""

import subprocess
#import shutil
import os
import argparse

parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--calc_prop', type=str, default="tadf")
parser.add_argument('--prop_calc_method', type=str, default="wb97xd")
parser.add_argument('--num_serviers', type=int, default=2)
parser.add_argument('--remote_wkdir', type=str, default="project")
parser.add_argument('--GXXXX', type=str, default="G0000")
parser.add_argument('--PXXXX', type=str, default="P0000")
parser.add_argument('--OXXXX', type=str, default="O0000")


args = parser.parse_args()

print("num_serviers: ", args.num_serviers, type(args.num_serviers))
NUM_SERVIERS = args.num_serviers

print(args.remote_wkdir, type(args.remote_wkdir))
remote_wkdir = args.remote_wkdir

print(args.calc_prop, type(args.calc_prop))
calc_prop = args.calc_prop

print(args.prop_calc_method, type(args.prop_calc_method))
prop_calc_method = args.prop_calc_method


print(args.GXXXX, type(args.GXXXX))
print(args.PXXXX, type(args.PXXXX))
print(args.OXXXX, type(args.OXXXX))
GXXXX = args.GXXXX
PXXXX = args.PXXXX
OXXXX = args.OXXXX



#-----------main func --------------------------------------------------------
#-----------------------------------------------------------------------------

#-------- remote servier working dir ------------------------------------------
remote_base_wkdir = 'tucy@node130:/home/tucy/Computation-node131/Gaussian16-jobs/SALAM/%s/'%remote_wkdir
remote_GXXXX_prop_jobs_dir = remote_base_wkdir  + GXXXX +  "/prop_jobs/" 

print("The remote_GXXXX_prop_jobs_dir:")
print(remote_GXXXX_prop_jobs_dir)


call_cp_pg006sh_modelcom_to_GXXXX_pick_mols_com_files_dir =  'cp   ./inputScripts/PG006__bat_g09log_to_propcom.sh   ./inputScripts/copy_com_chk_logs_to_modification_finished.sh   ./inputScripts/bat*.sh   ./inputScripts/g16*.sh  ./inputScripts/*-tadf-model-com    ./inputScripts/*-nlo-model-com   '  +  '  ./project/'  +  GXXXX   +  '/pick_mols/com-files/ '
status, output = subprocess.getstatusoutput( call_cp_pg006sh_modelcom_to_GXXXX_pick_mols_com_files_dir )
print("Copy inputScripts files. status, output = ", status, output)


#------------------------------------------------------------------------------
base_wkdir = os.getcwd()
print("base_wkdir: \n", base_wkdir)


GXXXX_pick_mols_com_files_dir = './project/'   +  GXXXX  + '/pick_mols/com-files/'
os.chdir( GXXXX_pick_mols_com_files_dir )
CWD = os.getcwd()
print("#CWD: \n", CWD)


print("Call ./copy_com_chk_logs_to_modification_finished.sh ")
copy_com_chk_logs_to_modification_finished_script = "./copy_com_chk_logs_to_modification_finished.sh  "
retcode = subprocess.call(copy_com_chk_logs_to_modification_finished_script, shell=True)
print(retcode)


status, output = subprocess.getstatusoutput('cp   ./PG006__bat_g09log_to_propcom.sh   ./*-tadf-model-com  ./*-nlo-model-com   ./bat*.sh   ./g16*.sh    ./finished/  ')
print("Copy pbs script and propcom files. status, output = ", status, output)

os.chdir('./finished/')
CWD = os.getcwd()
print("#CWD: \n", CWD)


print("Run ./PG006__bat_g09log_to_propcom.sh   prop-model-com  \n")

bat_g09log_to_propcom_script = "./PG006__bat_g09log_to_propcom.sh     "

wb97xd_tadf_model_com = "./wb97xd-tadf-model-com"
lcwpbe_tadf_model_com = "./lcwpbe-tadf-model-com"
camb3lyp_tadf_model_com = "./camb3lyp-tadf-model-com"
b3lyp_tadf_model_com = "./b3lyp-tadf-model-com"
cis_tadf_model_com = "./cis-tadf-model-com"

wb97xd_nlo_model_com = "./wb97xd-nlo-model-com"
#lcwpbe_nlo_model_com = "./lcwpbe-nlo-model-com"
camb3lyp_nlo_model_com = "./camb3lyp-nlo-model-com"
b3lyp_nlo_model_com = "./b3lyp-nlo-model-com"


call_bat_g09log_to_propcom_script_cis_tadf = bat_g09log_to_propcom_script + cis_tadf_model_com
call_bat_g09log_to_propcom_script_b3lyp_tadf = bat_g09log_to_propcom_script + b3lyp_tadf_model_com
call_bat_g09log_to_propcom_script_wb97xd_tadf = bat_g09log_to_propcom_script + wb97xd_tadf_model_com
call_bat_g09log_to_propcom_script_lcwpbe_tadf = bat_g09log_to_propcom_script + lcwpbe_tadf_model_com
call_bat_g09log_to_propcom_script_camb3lyp_tadf = bat_g09log_to_propcom_script + camb3lyp_tadf_model_com


call_bat_g09log_to_propcom_script_b3lyp_nlo = bat_g09log_to_propcom_script + b3lyp_nlo_model_com
call_bat_g09log_to_propcom_script_wb97xd_nlo = bat_g09log_to_propcom_script + wb97xd_nlo_model_com
#call_bat_g09log_to_propcom_script_lcwpbe_nlo = bat_g09log_to_propcom_script + lcwpbe_tadf_model_com
call_bat_g09log_to_propcom_script_camb3lyp_nlo = bat_g09log_to_propcom_script + camb3lyp_nlo_model_com


if ( (calc_prop == "tadf") and (prop_calc_method == "wb97xd") ):
    retcode = subprocess.call(call_bat_g09log_to_propcom_script_wb97xd_tadf, shell=True)
    print(retcode)
elif ( (calc_prop == "tadf") and (prop_calc_method == "lcwpbe") ):
    retcode = subprocess.call(call_bat_g09log_to_propcom_script_lcwpbe_tadf, shell=True)
    print(retcode)
elif ( (calc_prop == "tadf") and (prop_calc_method == "camb3lyp") ):
    retcode = subprocess.call(call_bat_g09log_to_propcom_script_camb3lyp_tadf, shell=True)
    print(retcode)
elif ( (calc_prop == "tadf") and (prop_calc_method == "b3lyp") ):
    retcode = subprocess.call(call_bat_g09log_to_propcom_script_b3lyp_tadf, shell=True)
    print(retcode)
elif ( (calc_prop == "tadf") and (prop_calc_method == "cis") ):
    retcode = subprocess.call(call_bat_g09log_to_propcom_script_cis_tadf, shell=True)
    print(retcode)
elif ( (calc_prop == "nlo") and (prop_calc_method == "wb97xd") ):
    retcode = subprocess.call(call_bat_g09log_to_propcom_script_wb97xd_nlo, shell=True)
    print(retcode)
elif ( (calc_prop == "nlo") and (prop_calc_method == "camb3lyp") ):
    retcode = subprocess.call(call_bat_g09log_to_propcom_script_camb3lyp_nlo, shell=True)
    print(retcode)
elif ( (calc_prop == "nlo") and (prop_calc_method == "b3lyp") ):
    retcode = subprocess.call(call_bat_g09log_to_propcom_script_b3lyp_nlo, shell=True)
    print(retcode)
else:
    print("parameter: calc_prop: either tadf or nlo; ") 
    print("           prop_calc_method: either wb97xd, lcwpbe, camb3lyp, b3lyp, or cis.")
    exit(1)

print("calc_prop = ", calc_prop)
print("prop_calc_method = ", prop_calc_method)

status, output = subprocess.getstatusoutput(' cp   ./bat*.sh   ./g16*.sh   ./com-files ')
print("Copy PBS Scripts files. status, output = ", status, output)

os.chdir('./com-files/')
print("Enter ./com-files/")

#-------- copy prop coms to  prop_jobs dir --------------------------------------
status, output = subprocess.getstatusoutput(' mkdir   ./prop_jobs   2>/dev/null ')
print("mkdir prop_jobs. status, output = ", status, output)

status, output = subprocess.getstatusoutput(' cp    ./bat*.sh    ./g16*.sh   ./cpd*.com   ./prop_jobs/   2>/dev/null')
print("Copy PBS Scripts files and cpd*.com to  prop_jobs.  status, output = ", status, output)


if (NUM_SERVIERS == 2):
    print("NUM_SERVIERS = ", NUM_SERVIERS)

    os.chdir('./prop_jobs/')
    print("Enter ./prop_jobs/")
    
    status, output = subprocess.getstatusoutput(' scp  cpd*.com  ' + remote_GXXXX_prop_jobs_dir )
    print("scp cpd*.com to remote_GXXXX_prop_jobs_dir. status, output = ", status, output)

else:
    print("NUM_SERVIERS = ", NUM_SERVIERS)


os.chdir(base_wkdir)
CWD = os.getcwd()
print("#CWD: \n", CWD)
#------------------------------------------------------------------------------


print("\n### End of program: PG006!\n")

